/***********************************************************************
 *
 *	Copyright (C) 1984-1995 Micro Focus Ltd.
 *
 *	All  rights reserved.  No part of this program or  publication
 *	may  be  reproduced,  transmitted,  transcribed,  stored  in a
 *	retrieval system,  or translated into any language or computer
 *	language, in any form or by any means, electronic, mechanical,
 *	magnetic,  optical, chemical, manual or otherwise, without the
 *	prior written permission of:
 *
 *	Micro Focus Ltd,
 *	26 West Street, Newbury, Berkshire, United Kingdom.             
 *	(0635) 32646
 *
 *	Phase6 COBOL driver
 *
 ************************************************************************/

static char sccsid[] = "@(#) cobrun.c 4.3";
/* Based on ph7 version 1.19 */

/*
 *    A helpful program to get us into the VS cobol system...
 *
 *    This program is used to call the run-time systems for Micro Focus
 *    VS COBOL.
 *
 *    The object file created from this source expects to be linked to
 *    other file names.
 *      cobrun  --- run and int code or generated code file
 *      anim    --- to animate a program compiled for animation.
 *
 *    The directory containing the cobol system may be specified by the
 *    environment variable COBDIR. If this is not set then a default value
 *    is used.
 *
 *    If Sourcewriter is to be installed this file should be linked to "swr".
 *    It will be important to note that this file assumes that Sourcewriter
 *    will be installed in /usr/lib/swr and that an swr.lib resides in this
 *    directory.
 */
   
#include	<stdio.h>
#include	<string.h>
#include	<sys/types.h>
#define		SYSTYPES
#include	<sys/stat.h>
#ifndef SOLARIS2
#include	<a.out.h>
#endif
#include	"cobrun.h" 

#define COBDELIMIT '='

/* Cobol system version, as found in file $COBDIR/cobver */
#define ET		1
#define VS_15		2
#define COBOL2_10	3
#define COBOL2_11	4
#define COBOL2_12	5
#define COBOL2_13	6
#define COBOL2_30	7
#define COBOL2_3R	8
#define COBOL2_3050	9
#define COBOL2_31	3100
#define COBOL2_32	3200
#define COBOL2_40	4000
#define COBOL2_41	4100

char *getenv();
char *env_val();

#define  TRUE      1
#define  FALSE     0

/* maximum number of arguments */
#define  ARGS      100 

/* define array sizes - Fix for NWB08082.  PIM.  05Mar91. */
#define PATHNAMLEN 256		/* Max size for a PATH name.  Was 120/128/200 */
#define ENVVARLEN 256		/* Max size for an environ var - Was 100/200 */
#define ARGUMENTLEN 256		/* Max size for an Argument.  Was 100 */

/* Run Time system to use with any command */
#define  RTS	   "rts32"

/* ET cbrun command*/
#define  ET_CBRUN  "/usr/bin/cbrun"
 
/* COBOL directory default */
# define COBOLDIR  DEFCOBDIR
char	*cmdbuff[ ARGS ];

char	**cmdargv = cmdbuff;

static char tmpbuf[PATHNAMLEN];

int mapsinit();
char *mkcobdir();
int swr_env();
char *optval();
char *swrpath();
char *setenv();
char *strstor();
int screens_env();
int editor_env();
int ds_env();

struct func {
	char *name;
	int pathtype;		/* 1 = $COBDIR; 0 = $PATH */
	int (*setup)();
	char *prog;
	char *options;
	char *helpmsg;
	int  arguments;		/* 0 = none, 1 = [options] */
				/* 2 = [options] filename  */
				/* 3 = [options] [rts-flags] filename */
				/* 4 = [options] [rts-flags] */
				/* 5 = [pass_options_to_cmdln] filename */
};

struct func cobrun   = { "cobrun",1,0,RTS,NULL,"run time system",3};
struct func fhcreate = { "fhcreate",1,0,RTS,"fhutil fhcreate","fhcreate",5};
struct func fhedit   = { "fhedit",1,0,RTS,"fhutil fhedit","fhedit",5};
struct func fhconvert= { "fhconvert",1,0,RTS,"fhutil fhconvert","fhconvert",5};
struct func fhrebuild= { "fhrebuild",1,0,RTS,"fhutil fhrebuild","fhrebuild",5};
struct func fhinfo   = { "fhinfo",1,0,RTS,"fhutil fhinfo","fhinfo",5};
struct func fhvalidate={ "fhvalidate",1,0,RTS,"fhutil fhvalidate","fhvalidate",5};
struct func fhreorg  ={ "fhreorg",1,0,RTS,"fhutil fhreorg","fhreorg",5};
struct func anim     = { "anim",1,0,"animator",NULL,"Animator",2};
struct func cobol    = { "cobol",1,0,"check",NULL,"compiler",2};
#ifdef att3b
struct func cgen     = { "cgen",0,0,"cob","-u","native code generator",2};
#else
struct func cgen     = { "cgen",1,0,"ncg",NULL,"native code generator",2};
#endif
struct func cobprof  = { "cobprof",1,0,"profiler",NULL,"profiler",2};
struct func forms2   = { "forms2",1,0,"forms2/FORMS2",NULL,"forms package",1};
struct func upgrade3 = { "upgrade3",1,0,"conversion/upgrade3",NULL,"RM source upgrade",1};
struct func convert3 = { "convert3",1,0,"conversion/convert3",NULL,"RM datafile upgrade",1};
struct func upgrade4 = { "upgrade4",1,0,"conversion/upgrade4",NULL,"Ti source upgrade",1};
struct func reform5 = { "reform5",1,0,"conversion/reform5",NULL,"DG source upgrade",1};
struct func convert5 = { "convert5",1,0,"conversion/convert5",NULL,"DG datafile upgrade",1};
#ifdef	MRI
struct func adiscf = { "adiscf",1,0,"adiscf.lbr/ADISCF",NULL,"adis configuration",1};
#else
struct func adiscf   = { "adiscf",1,0,"ADISCF",NULL,"adis configuration",1};
#endif
struct func keybcf   = { "keybcf",1,0,"KEYBCF",NULL,"keyboard configuration",1};
struct func maps     = { "maps",1,0,"comaps/MAPS",NULL,"Comaps",1};
struct func tabx     = { "tabx",1,0,"conversion/tabx",NULL,"RM tab convertor",5};
/* struct func swr      = { "swr",0,swr_env,"/usr/lib/swr/swr.lib","/usr/lib/swr/SWR.gnt", "sourcewriter",1}; */
struct func swr      = { "swr",0,swr_env,"senv",NULL, "sourcewriter",1};
struct func analyze = { "analyze",1,0,"analyze",NULL,"Analyzer package",1};
struct func screens   = { "screens",1,screens_env,"screens", NULL, "screens package",1};
struct func editor   = { "editor",1,editor_env,"edit", NULL, "editor package",1};
struct func tbox   = { "tbox",1,0,"tbox", NULL, "Toolbox driver",1};
struct func ds   = { "ds",1,ds_env,"dialog/DSC", NULL, "Dialog System",1};
struct func h2cpy   = { "h2cpy",1,0,"h2cpy", NULL, "header to copy utility",5};
struct func codecomp   = { "codecomp",1,0,"codecomp", NULL, "codeset modification",5};
struct func split78   = { "split78",1,0,"split78", NULL, "utility to split level 78s",5};
struct func fs   = { "fs",1,0,"fs", NULL, "Fileshare 2",5};
struct func animbase     = { "animbase",1,0,"animator",NULL,"low memory animator",2};
struct func mfsort     = { "mfsort",1,0,"mfsort",NULL,"mfsort",5};
struct func library = { "library",1,0,"tools.lbr/library",NULL,"Library facility",1};
struct func builder     = { "builder",1,0,"builder",NULL,"Co-Writer Builder",1};
struct func designer     = { "designer",1,0,"designer",NULL,"Co-Writer Designer",1};
struct func cwsource     = { "cwsource",1,0,"cwsource",NULL,"Co-Writer Sourcewriter Conversion",1};
struct func fsviewmot   = { "fsviewmot",1,0,"fsviewmot",NULL,"???",1};
struct func sstran	= { "sstran",1,0,"dialog/SSTRAN",NULL,"Screenset converter",1};
struct func browser   = { "browser",1,0,"browser", NULL, "Browser",0};
struct func infomgr   = { "infomgr",1,0,"infomgr", NULL, "infomgr",0};
struct func d   = { "d",1,0,"d", NULL, "Directory facility",0};
struct func cobdag   = { "cobdag",1,0,"cobdag", NULL, "IDL auto generator",5};
struct func cobidl   = { "cobidl",1,0,"cobidl", NULL, "COBOL IDL generator",5};
struct func mfinfocmp   = { "mfinfocmp",1,0,"mfinfocmp",NULL,"MF terminfo decompiler",2};
struct func mftic       = { "mftic",1,0,"mftic",NULL,"MF terminfo compiler",2};
struct func rtscore     = { "rtscore",1,0,"rtscore",NULL,"RTS core dump debugger",2};
struct func rebuild     = { "rebuild",1,0,"rebuild",NULL,"rebuild",5};
struct func oldh2cpy   = { "oldh2cpy",1,0,"oldh2cpy", NULL, "header to copy utility",5};
struct func daisgen   = { "daisgen",1,0,"gen.lbr/daisgen", NULL, "DAISGEN",3};

struct func *list[] = { 
&cobrun, &anim, &cobol, &cgen, &cobprof, &forms2, &upgrade3, &convert3, 
&fhcreate,&fhedit,&fhconvert,&fhrebuild,&fhvalidate,&fhinfo,&fhreorg,
&upgrade4, &reform5, &convert5, &adiscf, &maps, &tabx, &keybcf, &swr, &screens,
&analyze, &editor, &tbox, &ds, &h2cpy, &split78, &fs, &animbase, &mfsort,
&library, &builder, &designer, &cwsource, &fsviewmot, &sstran, &browser, &infomgr, &d,
&cobidl, &cobdag, &mfinfocmp, &mftic, &rtscore, &rebuild, &oldh2cpy,
&daisgen, &codecomp,

(struct func *)0 
};

main( argc, argv )
int	argc;
char	**argv ;

{	register char *s, *cpsw ;
	register char **avp;		/* argument vector pointer */
	int a;
	struct func *cmd;
	int i;
	char	*cmd_name;
	int	name_read = 0;
	short vflag =0 ;
	char *execname;
	static char compbuf[PATHNAMLEN];
	static char intname[PATHNAMLEN];
	static char cobsw[ENVVARLEN];

/* check to see if its ET or VS path */

	switch (getcobver())
	{
	    case ET:					/* not known */
	    	/* use the ET "cbrun" command to invoke the cobol system */
		execv( ET_CBRUN, argv);
		errmsg("cannot exec ", ET_CBRUN);
		break;
	    case VS_15:					/* vs 1.5 */
	    case COBOL2_10:				/* cobol\2 1.0 */
	    case COBOL2_11:				/* cobol\2 1.1 */
	    case COBOL2_12:				/* cobol\2 1.2 */
	    case COBOL2_13:				/* cobol\2 1.3 */
	    case COBOL2_3050:				/* cobol 3.0.XX */
	    case COBOL2_30:				/* cobol 3.0 */
	    case COBOL2_3R:				/* cobol 3.R */
	    case COBOL2_31:				/* cobol 3.1.XX */
	    case COBOL2_32:				/* cobol 3.2.XX */
	    case COBOL2_40:				/* cobol 4.0.XX */
	    case COBOL2_41:				/* cobol 4.1.XX */
		break;
	    default:
		errmsg("cobol version number invalid, call tech support ",NULL);
		break;
	}
	    

/* get last element of path name */
	if(cmd_name = strrchr(argv[0], '/'))
		cmd_name++;
	else
		cmd_name = argv[0];

/* set cmd according to name */
	for(i = 0; cmd = list[i]; i++)
		if (strcmp(cmd->name, cmd_name) == 0)
			break;

/* error if command name is not recognised */
	if(cmd == (struct func *)0 )
		errmsg("Invalid command name ", cmd_name);

	avp = cmdargv;

/* do any startup work */
	if ( cmd->setup != 0)
		cmd->setup();

/* does it require RTS support ?? */
	if ( cmd->pathtype == 0)
	{
	    execname=cmd->prog;
	    addarg( execname, avp++);		/* first arg is prog name */
	    cmd->prog=0;
	}
	else
	{
	    execname = mkcobdir( tmpbuf, cmd->prog);
	    if ( notaout( execname)	/* yep, then RTS must be exec'ed*/)
	    	execname = mkcobdir( tmpbuf,RTS);
	    else			/* otherwise, exec the prog	*/
		cmd->prog = 0;
	    addarg( execname, avp++);		/* argv0 is prog name */
	}

/* get env */
	strcpy( cobsw, "COBSW=");
	if ((cpsw = getenv("COBSW")) != (char *)0)
	    strcat( cobsw, cpsw);
	if(cmd == &animbase)
	    strcat( cobsw, "+A1");
#ifdef BASEANIMONLY
        if(cmd == &anim)
            strcat( cobsw, "+A1");
#endif

/* for each argument */
	a = 1;
	while (a < argc && ! name_read && cmd->arguments != 5)
	{
		s = argv[a++];

		if (*s  == '-')
			switch (s[1])
			{
				default:  strcat( cobsw, s); break;
				case 'v': vflag++ ; break;
				case 'h': help(cmd); return;
			}
		
		else if (*s == '+')
			strcat( cobsw, s);
		else
		/* first non-flag character read */
			name_read++;
	}

	putenv(cobsw);		 	/* copy rts control flags */

	if (cmd->prog)			/* if not yet done, get prog name */
	    addarg(mkcobdir(intname, cmd->prog), avp++);

	if (cmd->options) 	/*	 and then its options (if any) */
	{   char *malloc();
	    char *l, *s, *t = malloc(strlen(cmd->options) + 1);

	    s = cmd->options;

	    do
	    {	l = t;

	        while(*s && *s != ' ') *t++ = *s++;
		*t++ = '\0';
		addarg(l, avp++);
		while(*s && *s == ' ')
			s++;

	    } while(*s);
	}

	if (name_read)			/* add name to command list */
	{ 				/* s contains the name here */
	    if (cmd == &cobrun)		/* if we are running comms... */
	    	if (strcmp(s, "COMMS") == 0 || strcmp(s, "comms") == 0)
		    s = mkcobdir( compbuf, "comms");
	    addarg(s, avp++);		/* add name to list */
	}

	if(cmd == &anim || cmd == &animbase)
	{   if (!name_read)
		    errmsg("No file name given", NULL);
	    /* addarg("rnim", avp++); */
	}

	while (a < argc)		/* copy rest of arguments */
	{
#ifdef att3b
	    if (cmd == &cgen)
		addarg("-N",avp++);
#endif
	    addarg(optval(argv[a++]), avp++);
	}

	addarg( NULL, avp ) ;   /* terminate the list of args for execv */

/* Add to (or create) COBPATH so that $COBDIR/dialog is searched. This is
   neccessary not only for the 'ds' command, but also for user programs that
   make calls to DSRUN (ie the 'cobrun' and 'anim' commands).
*/
        {	register char *pathptr = env_val("COBPATH");
		register char *setptr;
		char tmpbuff[PATHNAMLEN]; /* yuk */

		char *malloc();
		char *strstor();

		if(pathptr == (char *)0)
			pathptr = "";

                mkcobdir(tmpbuff, "dialog");

		/* +10 == 1 for ':'. 1 for '\0'. 8 for "COBPATH=" */

		if((setptr = malloc(strlen(tmpbuff) + strlen(pathptr) + 10))
			== (char *)0)
		{
		    fprintf(stderr, "Failed to allocate enough memory\n");
		    exit(1);
		}

		strcpy(setptr, "COBPATH=");
		strcat(setptr, tmpbuff);
		strcat(setptr, ":");
		strcat(setptr, pathptr);

		putenv(strstor(setptr));

		free(setptr);
	}

	if (cmd == &infomgr)
        {       register char *pathptr = env_val("COBDATA");
                register char *setptr;
                char tmpbuff[PATHNAMLEN]; /* yuk */
 
                char *malloc();
                char *strstor();
 
                if(pathptr == (char *)0)
                        pathptr = "";
 
                mkcobdir(tmpbuff, "infomgr");
 
                /* +10 == 1 for ':'. 1 for '\0'. 8 for "COBDATA=" */
 
                if((setptr = malloc(strlen(tmpbuff) + strlen(pathptr) + 10))
                        == (char *)0)
                {
                    fprintf(stderr, "Failed to allocate enough memory\n");
                    exit(1);
                }
 
                strcpy(setptr, "COBDATA=");
                strcat(setptr, tmpbuff);
                strcat(setptr, ":");
                strcat(setptr, pathptr);
 
                putenv(strstor(setptr));
 
                free(setptr);
        }

/* if verbose flag is set, print out command line */
	if( vflag )
	{
		fputs( execname, stderr);
		for( avp = &cmdargv[1] ; *avp ; avp++ )
		{	fputs( " ", stderr);
			fputs( *avp, stderr) ;
		}
		fputs( "\n", stderr);
	 }

	if ( cmd->pathtype == 1)
	    execv( execname,  cmdargv) ;
	else
	    execvp( execname, cmdargv);

	errmsg("cannot exec ", execname);
}


/* add an argument to the argument list */
addarg( arg, avp )  char *arg ; char **avp ;
{	
	if( avp > &cmdargv[ ARGS ] ) 
	    errmsg("too many args", NULL);
	*avp = arg ;
}

/* give help */
help(cmd)
register struct func *cmd;

{
	fprintf( stderr, "%s: vs cobol %s driver program.\n", cmd->name,
			cmd->helpmsg);

	switch ( cmd->arguments)
	{
	    case 5:
		fprintf( stderr, "%s: usage: [utility options] filename\n",
			cmd->name); break;
	    case 4:
		fprintf( stderr, "%s: usage: [options] [rts-flags]\n",
			cmd->name); break;
	    case 3:
		fprintf( stderr, "%s: usage: [options] [rts-flags] filename\n",
			cmd->name); break;
	    case 2:
		fprintf( stderr, "%s: usage: [options] filename\n",
			cmd->name); break;
	    case 1:
		fprintf( stderr, "%s: usage: [options]\n",
			cmd->name); break;
	    default:
		return;
	}
	fputs( "\twhere options can be one of:\n\t\t-v  show the built command line\n\t\t-h  print this information\n", stderr);

}



/* give and error message and exit */
errmsg( msg, name )
	char *msg ;
	char *name ;
{
	fputs( "error: ", stderr);
	if (msg) fputs( msg, stderr ) ;
	if (name) fputs( name, stderr ) ;
	fputs( "\n", stderr);
	exit( 1 ) ;
}


/***************************************************************************
 * mkcobdir - given a basename add the appropriate COBDIR to it and return
 * a pointer to the full name.
 */

char *
mkcobdir(cdbuf, name)
	char *cdbuf;
	char *name;

{	char *env_val();
	register char *cp;
	register char *tp = cdbuf;

	if ((cp = env_val("COBDIR")) == (char *)0)
	{
		cp = COBOLDIR;                  /* assume default directory */
	/* Fix for nwb13554 by ST, 07apr92 */
		setenv("COBDIR", COBOLDIR);
	}

	while (*tp = *cp) 			/* copy directory */
	{	tp++;
		cp++;
	}
	if ( tp > cdbuf && *(tp - 1) != '/')	/* terminate */
		*tp++ = '/';
	while (*tp++ = *name++) ;		/* copy name */
	return( cdbuf);
}


/* determine if progname is an a.out file */

notaout( progname)
char *progname;
{
	unsigned short head;
	int lfd;
	int readsz;

	if (( lfd = open( progname, 0)) == -1)	/* open the potential a.out */
		return( TRUE);
	readsz=read( lfd, (char *)&head, sizeof(head));
	close( lfd);
						/* check its magic number   */
	if (readsz != sizeof(head) || ! ISMAG(head))
		return( TRUE);
	return( FALSE);
}


/* determine if COBDIR is an ET or VS system */
/* we can always count on there being a Cobol compiler in the Cobol */
/* directory, by determining if it is an ET or VS version we can    */
/* guess at the rest of the current Cobol system.		    */

is_et_path()
{
	char buff[PATHNAMLEN];
	char *cp;
	char *buffend;

	if ((cp = env_val("COBDIR")) != (char *)0)
	    strcpy( buff, cp);
	else
	    strcpy( buff, COBOLDIR);     /* assume default directory */

	strcat( buff, "/");
	buffend = buff + strlen( buff);	/* dir to search in buff */

	/* now try to find the Compiler */
	/* note, we use access amode=00 because we are only trying to see */
	/* what type of system we have; if the file does not have the	  */
	/* correct permissions the correct error messge will be produceed */
	/* later.							  */

	strcpy( buffend, "check");	/* is there a  VS compiler ? */
	if ( access( buff, 00) == 0)
	    return( VS_15);

	strcpy( buffend, "COBOL.GNT");	/* is there an ET compiler ? */
	if ( access( buff, 00) == 0)
	    return( ET);
	strcpy( buffend, "cobol.GNT");	/* is there an ET compiler ? */
	if ( access( buff, 00) == 0)
	    return( ET);
	strcpy( buffend, "COBOL.INT");	/* is there an ET compiler ? */
	if ( access( buff, 00) == 0)
	    return( ET);
	strcpy( buffend, "cobol.INT");	/* is there an ET compiler ? */
	if ( access( buff, 00) == 0)
	    return( ET);

	*buffend='\0';
	errmsg("cannot find Cobol compiler at: ", buff);
}

/******************************************************************************
*                               swr_env()                                     *
*   This routine sets up the MFPATH, COBCAP and DD_ADISCTRL environment       *
*   variables used by SOURCEWRITER.                                           *
*       If these environment vars are not already present, new                *
*   values will be created from COBDIR or from the default cobol library.     *
*   The new values are then part of the environ passed to child processes.    */
int      
swr_env()
{
    char *cobsw;

    if (env_val("MFPATH") == NULL )
	setenv("MFPATH", swrpath("MFPATH"));

    if (env_val("COBCAP") == NULL )
	setenv("COBCAP", swrpath("COBCAP"));

    if (env_val("DD_ADISCTRL") == NULL && env_val("dd_ADISCTRL") == NULL )
	setenv("DD_ADISCTRL" ,swrpath("ADISCTRL"));

    if (env_val("DD_cobkeymp") == NULL  && env_val("dd_cobkeymp") == NULL )
	setenv("DD_cobkeymp", swrpath("cobkeymp"));

    if ((cobsw = env_val("COBSW")) != NULL )   /* COBSW already set ? */
    {						/* keep current settings */
	char *s1;
	unsigned size;
	char *malloc();

	size = strlen(cobsw) + 2 + 1;		/* COBSW + "-F" + '\0' */
	if ((s1 = malloc(size)) == 0)
	{
	    fprintf(stderr, "failed to allocate %d bytes of memory\n", size);
	    exit(1);
	}
	cobsw = strcpy(s1, cobsw);		/* Bung "-F" on end */
        strcat(cobsw, "-F");
    }
    else
	cobsw = "-F";
    setenv("COBSW", cobsw);
}

static char *swrdir;			/* SWR home dir */
/*
 * swrpath() - given last component of swr relative path, make name absolute.
 */
char *
swrpath(dir)
char *dir;
{
    if (swrdir == 0)			/* if SWRPATH not know find it */
    {
	if ((swrdir = env_val("SWRDIR")) == NULL )
	    swrdir = "/usr/lib/swr";	/* default to /usr/lib/swr */
    }
    strcpy(tmpbuf, swrdir);
    strcat(tmpbuf, "/");
    strcat(tmpbuf, dir);

    return tmpbuf;
}


/*
 * env_val() - get name from environment
 */
char *
env_val(name)
char *name;
{
    char *env;
    if ( ( (env = getenv(name) ) != (char *)0 ) && (*env != '\0') )
	return env;

    return (char *)0;
}

/*
 * setenv() - put name=value into environment
 */
char *
setenv(name, val)
char *name;
char *val;
{
    char env_var[ENVVARLEN];

    strcpy(env_var, name);
    strcat(env_var, "=");
    strcat(env_var, val);
    if (putenv(strstor(env_var)) != 0)
	errmsg("Failed to malloc space for environment vars", NULL);
}

/* allocate space, copy the string into it, return pointer to space */
char *
strstor(s2)
char *s2;
{
    char *s1;
    unsigned size;
    char *malloc();

    size = strlen(s2) + 1;
    if ((s1 = malloc(size)) == 0)
    {
	fprintf(stderr, "failed to allocate %d bytes of memory\n", size);
	exit(1);
    }
    return(strcpy(s1, s2));
}


/*
*  given an argument for COBOL or NCG, map first '=' to '(' and append ')'
*/
char *
optval(next)
register char *next;
{
    register char *work;
    char wrk[ARGUMENTLEN];

    wrk[0]='\0';
    work = next;
    while (work != NULL && (work = strchr(next, COBDELIMIT)) != NULL)
    {
	if (work[1] == COBDELIMIT)
	{				/* allow COBDELIMIT to be itself */
	    strncat(wrk, next, work-next+1);
	    next = work + 2;
	}
	else
	{				/* map xxCOBDELIMITyy ->  xx(yy) */
	    strncat(wrk, next, work-next);
	    strcat(wrk, "(");
	    next = work+1;
	    while (*next != '\0' && *next == ' ') next++;
	    if ((work = strchr(next, ' ')) != NULL)
		strncat(wrk, next, work-next);
	    else
		strcat(wrk, next);
	    strcat(wrk, ")");
	    next = work;
	}
    }
    if (next)
	strcat(wrk, next);
    return(strstor( wrk));
}

/*
 * getcobver - return Cobol version number
 */
getcobver()
{
    char *cobver;
    FILE *fver;
    char *buff,*s;
    char tmpbuf[PATHNAMLEN];
    struct stat st;
    int size, readsz;
    char *malloc();

    cobver = mkcobdir( tmpbuf, "cobver");
    if ((fver = fopen(cobver, "r")) == (FILE *)NULL)
	return is_et_path();
    if (stat(cobver, &st) != 0)
	errmsg( "cannot stat: %s\n", cobver);
    size = st.st_size + 1;
    if ((buff = malloc(size)) == 0)
    {
	fprintf(stderr, "failed to allocate %d bytes of memory\n", size);
	exit(1);
    }
    readsz = fread(buff, 1, (int)size, fver);
    fclose(fver);
    if (readsz != st.st_size)
    {
	errmsg("partial read on: %s\n", cobver);
    }
    buff[readsz] = '\0';

    for (s = buff; *s != '\0'; )
    {
	if (strncmp(s, "et", 2) == 0)
	    return ET;
	else if (strncmp(s, "vs15", 4) == 0)
	    return VS_15;
	else if (strncmp(s, "cobol/2 v1.0", 12) == 0)
	    return COBOL2_10;
	else if (strncmp(s, "cobol/2 v1.1", 12) == 0)
	    return COBOL2_11;
	else if (strncmp(s, "cobol/2 v1.2", 12) == 0)
	    return COBOL2_12;
	else if (strncmp(s, "cobol/2 v1.3", 12) == 0)
	    return COBOL2_13;
	else if (strncmp(s, "cobol v3.1.", 11) == 0)
	    return COBOL2_31;
	else if (strncmp(s, "cobol v3.2.", 11) == 0)
	    return COBOL2_32;
	else if (strncmp(s, "cobol v4.0.", 11) == 0)
	    return COBOL2_40;
	else if (strncmp(s, "cobol v4.1.", 11) == 0)
	    return COBOL2_41;
	else if (strncmp(s, "cobol v3.0.", 11) == 0)
	    return COBOL2_3050;
	else if (strncmp(s, "cobol v3.0", 10) == 0)
	    return COBOL2_30;
	else if (strncmp(s, "cobol v3.R", 10) == 0)
	    return COBOL2_3R;
	else
	    s++;
    }
    return 0;
}
int screens_env()
{
    char *cobsw;

    if ((cobsw = env_val("COBSW")) != (char *)0 ) /* COBSW already set ? */
    {						/* keep current settings */
	char *s1;
	unsigned size;
	char *malloc();

	size = strlen(cobsw) + 2 + 1;		/* COBSW + "-i" + '\0' */
	if ((s1 = malloc(size)) == 0)
	{
	    fprintf(stderr, "failed to allocate %d bytes of memory\n", size);
	    exit(1);
	}
	cobsw = strcpy(s1, cobsw);		/* Bung "-i" on end */
        strcat(cobsw, "-i");
    }
    else
	cobsw = "-i";
    setenv("COBSW", cobsw);
    return 0;
}

int editor_env()
{
    char *cobsw;

    if ((cobsw = env_val("COBSW")) != (char *)0 ) /* COBSW already set ? */
    {						/* keep current settings */
	char *s1;
	unsigned size;
	char *malloc();

	size = strlen(cobsw) + 2 + 1;		/* COBSW + "-i" + '\0' */
	if ((s1 = malloc(size)) == 0)
	{
	    fprintf(stderr, "failed to allocate %d bytes of memory\n", size);
	    exit(1);
	}
	cobsw = strcpy(s1, cobsw);		/* Bung "-i" on end */
        strcat(cobsw, "-i");
    }
    else
	cobsw = "-i";
    setenv("COBSW", cobsw);
    return 0;
}

int ds_env()
{
    char *cobsw;
    char tmpbuff[PATHNAMLEN];

    if ((cobsw = env_val("COBSW")) != (char *)0 ) /* COBSW already set ? */
    {						/* keep current settings */
	char *s1;
	unsigned size;
	char *malloc();

	size = strlen(cobsw) + 2 + 1;		/* COBSW + "-i" + '\0' */
	if ((s1 = malloc(size)) == 0)
	{
	    fprintf(stderr, "failed to allocate %d bytes of memory\n", size);
	    exit(1);
	}
	cobsw = strcpy(s1, cobsw);		/* Bung "-i" on end */
        strcat(cobsw, "-i");
    }
    else
	cobsw = "-i";
    setenv("COBSW", cobsw);

    /* Ensure COBHELP is set appropriately for DS */
    setenv("COBHELP", mkcobdir(tmpbuff, "dialog/help"));

    return 0;
}
