      *>----------------------------------------------------------------
      *> A CallBack stores a message which may be Invoked later. Send
      *> the message "invoke" to a CallBack to invoke the message
      *> stored. Note the maximum size of a message is 80 bytes.
      *>
      *> Copyright (c) Micro Focus Ltd 1991, 1996
      *>
      *>----------------------------------------------------------------
      $set mfoo
       class-id. callback data is protected
               inherits from EntryCallback
                  with data.

        object section.
        class-control.
           EntryCallback is class "entrycll"
           CharacterArray is class "chararry"
           callback is class "callback".
       working-storage section.

       object-storage section.
      *>----------------------------------------------------------------
      *> Return an instance of CallBack with the receiver aReceiver,
      *> the message selector aMessageName and p1 - p6 extra parameters.
      *> The addresses of the parameters are stored and passed
      *> preceeding the actual parameters when this CallBack is is
      *> Invoked. If more than six extra parameters are specified
      *> then an exception is raised.
      *>----------------------------------------------------------------
       method-id. "new".
       local-storage section.

       01 exceptionnumber          pic s9(9) comp-5.
       01 sendObjectHandle         usage object reference.
       01 messageText              pic x(80).
       01 parmPointer              usage pointer.
       01 serverid                 pic x(4) comp-5.

       linkage section.
       01 p1                   usage object reference.
       01 p2                   usage object reference.
       01 p3                   usage object reference.
       01 p4                   usage object reference.
       01 p5                   usage object reference.
       01 p6                   usage object reference.
       01 p7notallowed         usage object reference.
       01 areceiver            usage object reference.
       01 result               usage object reference.
       01 amessagename         pic x(80).
       01 parmDescriptor       pic x.

       procedure division using areceiver amessagename
                                       p1 p2 p3 p4 p5 p6
                                       p7notallowed
                                       returning result.
           if address of p7notallowed not = null
               move 20 to exceptionnumber
               invoke self "raiseException" using exceptionnumber
                                             returning result
               exit method
           else
               invoke super "basicnew" returning result
               invoke result "initialize" using areceiver
                                                   amessagename
                                                   p1
                                                   p2
                                                   p3
                                                   p4
                                                   p5
                                                   p6

           end-if
           exit method.
       end method "new".
       Object.

       object-storage section.


       01 receiver             usage object reference.



       01 cacheBlock           pic x(4).

       01 hashValue            pic x(4) comp-5.

      *>----------------------------------------------------------------
      *> PRIVATE. Initialize the new instance of this callback.
      *>----------------------------------------------------------------

       method-id. "initialize".
       local-storage section.
       01 i pic x(4) comp-5.
       01 j                    pic x(4) comp-5.
       01 aString  usage object reference.
       01 aMask                pic x(4) comp-5.
       01 hashDigit            pic x.
       01 hashDigitAsNum redefines hashDigit pic x comp-5.

       linkage section.
       01 p1                   usage object reference.
       01 p2                   usage object reference.
       01 p3                   usage object reference.
       01 p4                   usage object reference.
       01 p5                   usage object reference.
       01 p6                   usage object reference.
       01 areceiver            usage object reference.
       01 amessagename         pic x(80).

       procedure division using areceiver amessagename p1 p2
                                                       p3 p4 p5 p6.
           move zero to numParams
           if address of p1 not = null
               set objectParms(1) to p1
               move 1 to numParams
               if address of p2 not = null
                   set objectParms(2) to p2
                   move 2 to numParams
                   if address of p3 not = null
                       set objectParms(3) to p3
                       move 3 to numParams
                       if address of p4 not = null
                           set objectParms(4) to p4
                           move 4 to numParams
                           if address of p5 not = null
                               set objectParms(5) to p5
                               move 5 to numParams
                               if address of p6 not = null
                                   set objectParms(6) to p6
                                   move 6 to numParams
                               end-if
                           end-if
                       end-if
                   end-If
               end-if
           end-if
           move spaces to messagename
           invoke characterArray "withSpaceOrNullTerminator"
                       using amessageName
                   returning aString
           invoke aString "sizeInBytes" returning i
           if i > 79
               move "callbackmethodnametoolong" to messageName
           else
               invoke aString "getValue" returning messageName
               call "CBL_TOLOWER" using messageName by value i
           end-if
           move zero to hashValue
           perform varying j from 1 by 1 until j > i
               multiply 2 by hashValue
               move messageName(j:1) to hashDigit
               add hashDigitAsNum to hashValue
           end-perform
           move h"FFFF" to aMask
           call "CBL_AND" using aMask
                                hashValue
                                by value 4
           invoke aString "finalize" returning aString
           set receiver to areceiver
           move zero to isInvoked
           exit method.
       end method "initialize".
      *>----------------------------------------------------------------
      *> Determine how many parameters there are and invoke the method
      *> with that number of parameters. The number of parameters is
      *> determined by calculating the sum of the optional parameters
      *> passed when this CallBack was instantiated (that is with the
      *> "new" message) and the number of parameters passed on the
      *> invoke message.
      *> The maximum number of parameters that can be passed with the
      *> invoke message is three, if any more are passed then an
      *> exception will be raised.
      *> The parameters passed on this call (anarg1, anarg2, and
      *> anarg3) will come after those passed at instantiation.
      *> The result of the method executed is returned in lnkreturnparam
      *>----------------------------------------------------------------

       method-id. "invoke".
       local-storage section.
       01 procPtr                  procedure-pointer.
       01 lsObject                 usage object reference.
       01 ooreturnptr              usage object reference.
       01 exceptionnumber          pic s9(9) comp-5.
       01 thereceiver              usage object reference.
       01 numparamsplusargs        pic s9(9) comp-5.
       01 i                        pic x(4) comp-5.
       01 theParameters occurs maxnumparams.
          05 parm              usage pointer.

       linkage section.
       01 lnkreturnparam            usage object reference.
       01 anarg1               usage object reference.
       01 anarg2               usage object reference.
       01 anarg3               usage object reference.
       01 arg4notallowed       usage object reference.

       procedure division using anarg1 anarg2 anarg3
                            arg4notallowed
                            returning lnkreturnparam.
           if address of lnkreturnparam = null
               set address of lnkreturnparam to address of ooreturnptr
           end-if
           if address of arg4notallowed not = null
               move 21 to exceptionnumber
               invoke self "raiseException" using exceptionnumber
                                             returning lnkreturnparam
           else
               perform varying i from 1 by 1 until i > numParams
                   set parm(i) to address of objectParms(i)
               end-perform
               move numparams to numparamsplusargs
               if address of anarg1 not = null
                   if receiver equal to null
                       set thereceiver to anarg1
                   else
                       set thereceiver to receiver
                       add 1 to numparamsplusargs
                       set parm(numparamsplusargs) to address of anarg1
                   end-if
                   if address of anarg2 not = null
                       add 1 to numparamsplusargs
                       set parm(numparamsplusargs) to address of anarg2
                       if address of anarg3 not = null
                           add 1 to numparamsplusargs
                           set parm(numparamsplusargs) to
                                                      address of anarg3
                       end-if
                   end-if
               else
                   set thereceiver to receiver
               end-if
               add 1 to isInvoked

               *>>> Warning... Following 2 calls are private and should
               *>>> not be used by any other class / programmer as
               *>>> implementation is likely to change.

               call "oopsresolve" using by reference messageName
                                        by reference theReceiver
                                        by value     hashValue
                                        by reference cacheBlock
                              returning procPtr

               evaluate numparamsplusargs
               when 0
               call procPtr using lnkreturnparam
               when 1
               call procPtr using lnkreturnparam by value parm(1)
               when 2
               call procPtr using lnkreturnparam by value parm(1)
                                                 by value parm(2)
               when 3
               call procPtr using lnkreturnparam by value parm(1)
                                                 by value parm(2)
                                                 by value parm(3)
               when 4
               call procPtr using lnkreturnparam by value parm(1)
                                                 by value parm(2)
                                                 by value parm(3)
                                                 by value parm(4)
               when 5
               call procPtr using lnkreturnparam by value parm(1)
                                                 by value parm(2)
                                                 by value parm(3)
                                                 by value parm(4)
                                                 by value parm(5)
               when 6
               call procPtr using lnkreturnparam by value parm(1)
                                                 by value parm(2)
                                                 by value parm(3)
                                                 by value parm(4)
                                                 by value parm(5)
                                                 by value parm(6)
               when 7
               call procPtr using lnkreturnparam by value parm(1)
                                                 by value parm(2)
                                                 by value parm(3)
                                                 by value parm(4)
                                                 by value parm(5)
                                                 by value parm(6)
                                                 by value parm(7)
               when 8
               call procPtr using lnkreturnparam by value parm(1)
                                                 by value parm(2)
                                                 by value parm(3)
                                                 by value parm(4)
                                                 by value parm(5)
                                                 by value parm(6)
                                                 by value parm(7)
                                                 by value parm(8)
               when 9
               call procPtr using lnkreturnparam by value parm(1)
                                                 by value parm(2)
                                                 by value parm(3)
                                                 by value parm(4)
                                                 by value parm(5)
                                                 by value parm(6)
                                                 by value parm(7)
                                                 by value parm(8)
                                                 by value parm(9)
               end-evaluate
               subtract 1 from isInvoked
               if requestFinalize and isInvoked = 0
                   invoke self "finalize" returning lsObject
               end-if
           end-if
           exit method.
       end method "invoke".
      *>----------------------------------------------------------------
      *> Return the receiver.
      *>----------------------------------------------------------------
       method-id. "getReceiver".
       linkage section.
       01 lnkObject         usage object reference.
       procedure division returning lnkObject.
           set lnkObject to receiver
           exit method.
       end method "getReceiver".


      *>----------------------------------------------------------------
      *> Returns true if self = lnkCallback
      *>----------------------------------------------------------------
       method-id. "equal".
       local-storage section.
       01 lsMessage          pic x(81).
       01 lsReceiver         object reference.

       linkage section.
       01 lnkCallback        object reference.
       01 lnkBool            pic x comp-5.
          88 isTrue          value 1.
          88 isFalse         value 0.

       procedure division using lnkCallback returning lnkBool.
           invoke lnkCallback "getMessage" returning lsMessage
           invoke lnkCallback "getReceiver" returning lsReceiver
           if messageName = lsMessage and receiver = lsReceiver
               set isTrue to true
           else
               set isFalse to true
           end-if
           exit method.
       end method "equal".
      *>----------------------------------------------------------------
      *> Reset the cache. Not valid on a second run.
      *>----------------------------------------------------------------

       method-id. "prepareToBeSaved".
       procedure division.
           move low-values to cacheBlock
           exit method.
       end method "prepareToBeSaved".

      *>----------------------------------------------------------------
      *> Destroys the object to which this message is sent. The result
      *> is always set to null. Therefore invoke anObject "finalize"
      *> returning anObject will destroy anObject and set the reference
      *> to null.
      *>----------------------------------------------------------------
       method-id. "finalize".
       linkage section.
       01 lnkObject        usage object reference.
       procedure division returning lnkObject.
           if isInvoked = 0
                call "oopsfreecache" using by value cacheBlock
                invoke super "finalize" returning lnkObject
           else
                set requestFinalize to true
           end-if
           exit method.
       end method "finalize".

       end object.
       end class callback.
