      *>----------------------------------------------------------------
      *> A CharacterArray stores strings as objects. It differs from
      *> CobolPicx as there is no need to clone a class to create
      *> instances of different sizes.
      *> Instead, CharacterArrays use indexed data and are therefore
      *> more efficient when creating instances. Use instances of
      *> Cobolpicx only when it is important that the instance data
      *> comprises just the string.
      *>
      *> Copyright (C) 1991,1996 Micro Focus Ltd
      *>
      *>----------------------------------------------------------------
      $set mfoo linkcount(512) localCount(512)
       class-id. characterarray data is protected
               inherits from array with data.


        object section.
        class-control.
           array is class "array"
           cobolpicx is class "picx"
           characterarray is class "chararry"
           ValueSet is class "valueset"
           DBCSCharacterArray is class "dbcsarry"
           Orderedcollection is class "ordrdcll".

       working-storage section.
       01 dbcsEnvironmentStatus    pic x comp-5 external.
           88  isSBCSEnv value 1.
           88  isDBCSEnv value 2.

       object-storage section.

      *>----------------------------------------------------------------
      *> Overrides the superclass method and raises an exception as
      *> ofReferences is not a valid message for character arrays.
      *> This means that the string can only compromise of actual
      *> characters and not references to characters.
      *>----------------------------------------------------------------
       method-id. "ofReferences".
       linkage section.
       01 lnksize           pic s9(9) comp-5.
       01 lnkobject         usage object reference.

       procedure division using lnksize returning lnkobject.
           invoke self "shouldNotImplement" returning lnkobject
           exit method.
       end method "ofReferences".
      *>----------------------------------------------------------------
      *> Creates a new character array of length lnkSize and value
      *> indicated by lnkValue. lnkSize is specified in bytes.
      *>----------------------------------------------------------------
       method-id. "withLengthValue".
       linkage section.
       01 lnksize           pic s9(9) comp-5.
       01 lnkobject         usage object reference.
       01 lnkvalue.
           03 aByte pic x occurs 1 to maxNumberBytes depending on
                                                       lnkSize.

       procedure division using lnksize lnkvalue
                            returning lnkobject.
           invoke self "new" using lnkSize returning lnkobject
           invoke lnkobject "setValue" using lnkvalue
           exit method.
       end method "withLengthValue".

      *>----------------------------------------------------------------
      *> Creates a new character array of length lnkSize and value
      *> indicated by lnkValue. lnkSize is specified in bytes.
      *>----------------------------------------------------------------
       method-id. "withByteLengthValue".
       local-storage section.
       01 lsBool   pic x comp-5.

       linkage section.
       01 lnksize           pic s9(9) comp-5.
       01 lnkobject         usage object reference.
       01 lnkvalue.
           03 aByte pic x occurs 1 to maxNumberBytes depending on
                                                       lnkSize.

       procedure division using lnksize lnkvalue
                            returning lnkobject.
           if isSBCSEnv
               invoke self "withLengthValue" using lnkSize lnkValue
                                     returning lnkobject
           else
               invoke DBCSCharacterArray "isDBCSStringByteLengthValue"
                       using lnkSize lnkValue
                   returning lsBool
               if lsBool = 0
                   invoke self "withLengthValue" using lnkSize lnkValue
                                     returning lnkobject
               else
                   invoke DBCSCharacterArray "withByteLengthValue"
                                         using lnkSize lnkValue
                                     returning lnkobject
               end-if
           end-if
           exit method.
       end method "withByteLengthValue".

      *>----------------------------------------------------------------
      *> Creates a new character array of length lnkSize. lnkSize is
      *> specified in bytes.
      *>----------------------------------------------------------------
       method-id. "withLength".
       linkage section.
       01 lnksize           pic s9(9) comp-5.
       01 lnkobject         usage object reference.

       procedure division using lnksize returning lnkobject.
           invoke self "new" using lnkSize returning lnkobject
           exit method.
       end method "withLength".
      *>----------------------------------------------------------------
      *> Create a characterArray from a null terminated string
      *> lnkString.
      *>----------------------------------------------------------------
       method-id. "withValue".
       local-storage section.
       01 i            pic x(4) comp-5.
       01 aString      usage object reference.
       linkage section.
       01 lnkstring    pic x.
       01 lnkObject    usage object reference.

       procedure division using lnkstring returning lnkObject.
           if isSBCSEnv
               perform varying i from 1 by 1 until lnkString(i:1) =
                   x"00"
               end-perform
               subtract 1 from i
               invoke self "withLengthValue" using i lnkString
                       returning lnkObject
           else
               invoke DBCSCharacterArray "withValue" using lnkString
                   returning lnkObject
           end-if
           exit method.
       end method "withValue".
      *>----------------------------------------------------------------
      *> Create a characterArray from a space terminated string
      *> lnkString.
      *>----------------------------------------------------------------
       method-id. "withSpaceTerminator".
       local-storage section.
       01 i         pic x(4) comp-5.
       01 aString   usage object reference.
       linkage section.
       01 lnkstring pic x.
       01 lnkObject usage object reference.

       procedure division using lnkstring returning lnkObject.
           if isSBCSEnv
               perform varying i from 1 by 1
                   until lnkString(i:1) = space
               end-perform
               subtract 1 from i
               invoke self "withLengthValue" using i lnkString
                   returning lnkObject
           else
               invoke DBCSCharacterArray "withSpaceTerminator"
                           using lnkString
                       returning lnkObject
           end-if
           exit method.
       end method "withSpaceTerminator".
      *>----------------------------------------------------------------
      *> Create a characterArray from a space or null terminated string
      *> lnkString.
      *>----------------------------------------------------------------
       method-id. "withSpaceOrNullTerminator".
       local-storage section.
       01 i         pic x(4) comp-5.
       01 aString   usage object reference.
       linkage section.
       01 lnkstring pic x.
       01 lnkObject usage object reference.

       procedure division using lnkstring returning lnkObject.
           if isSBCSEnv
               perform varying i from 1 by 1
                   until lnkString(i:1) = space
                   or lnkString(i:1) = x"00"
               end-perform
               subtract 1 from i
               invoke self "withLengthValue" using i lnkString
                       returning lnkObject
           else
               invoke DBCSCharacterArray "withSpaceOrNullTerminator"
                           using lnkString
                       returning lnkObject
           end-if
           exit method.
       end method "withSpaceOrNullTerminator".
      *>----------------------------------------------------------------
      *> Creates a new character array of length lnkSize with undefined
      *> contents. lnkSize is the size in bytes.
      *>----------------------------------------------------------------
       method-id. "new".

       local-storage section.
       01 astride              pic s9(9) comp-5.
       01 newSize              pic x(4) comp-5.
       linkage section.
       01 lnksize              pic s9(9) comp-5.
       01 lnkobject            usage object reference.

       procedure division using lnksize returning lnkobject.
           if lnksize < zero
               invoke self "invalidParameter" returning lnkobject
           else
               move 1 to astride
               add 1 to lnkSize giving newSize *> For null on the end
               call "oopsnewindexedinstance" using self newsize
                                         returning lnkObject
               invoke lnkObject "initialize" using lnkSize
                                                   cobolpicx
                                                   aStride
           end-if
           exit method.
       end method "new".
      *>----------------------------------------------------------------
      *> Overrides the superclass method ofValues to enforce that the
      *> template is CobolPICX. In fact the parameter lnkTemplate is not
      *> used.
      *>----------------------------------------------------------------
       method-id. "ofValues".
       linkage section.
       01 lnksize           pic s9(9) comp-5.
       01 lnktemplate       usage object reference.
       01 lnkobject         usage object reference.

       procedure division using lnktemplate lnksize
                           returning lnkobject.
           invoke self "new" using lnkSize returning lnkobject
           exit method.
       end method "ofValues".
      *>----------------------------------------------------------------
      *> Create a new string which contains the same characters as
      *> the character array lnkObject
      *>----------------------------------------------------------------
       method-id. "withAll".
       linkage section.
       01 lnkobject         usage object reference.
       01 lnknewstring      usage object reference.

       procedure division using lnkobject returning lnknewstring.
           invoke lnkobject "basiccopy" returning lnknewstring.
           exit method.
       end method "withAll".
       Object.

       object-storage section.
      *>----------------------------------------------------------------
      *> Return True if the value indicated by compareValue and
      *> compareLength is Equal to the value of this CharacterArray
      *> else return False.
      *>----------------------------------------------------------------
       method-id. "equalbyLengthValue".
       linkage section.
       01 comparevalue.
           03 abyte      pic x occurs 1 to maxnumberbytes depending on
                                   comparelength.
       01 lnkbool        pic x comp-5.
           88  lnkistrue  value 1.
           88  lnkisfalse value 0.
       01 comparelength        pic x(4) comp-5.

       procedure division using comparelength comparevalue
                                            returning lnkbool.
           if comparevalue = instancedata(indexeddataoffset + 1:
                                            storagerequirements)
               set lnkistrue to true
           else
               set lnkisfalse to true
           end-if
           exit method.
       end method "equalbyLengthValue".
      *>----------------------------------------------------------------
      *> Return 1 if this characterArray is greater than anArray.
      *> Return -1 if this characterArray is less than anArray.
      *> Return 0 if this characterArray is equal to anArray.
      *>----------------------------------------------------------------
       method-id. "compareTo".
       local-storage section.

       01 comparelength       pic x(4) comp-5.
       01 aptr                usage pointer.

       linkage section.
       01 comparevalue.
           03 abyte           pic x occurs 1 to maxnumberbytes
                                       depending on comparelength.
       01 anarray             usage object reference.
       01 lnkResult           pic s9(9) comp-5.
           88  lnkGreaterThan value  1.
           88  lnkLessThan    value -1.
           88  lnkEqual       value  0.

       procedure division using anarray returning lnkResult.
           invoke anarray "sizeInBytes" returning compareLength
           invoke self "malloc" using compareLength returning aptr
           set address of comparevalue to aptr
           invoke anarray "getValue" returning compareValue
           if instancedata(indexeddataoffset + 1:
                             storagerequirements)  < comparevalue
               set lnkLessThan to true
           else
               if instancedata(indexeddataoffset + 1:
                             storagerequirements) > comparevalue
                   set lnkGreaterThan to true
               else
                   set lnkEqual to true
               end-if
           end-if
           invoke self "dealloc" using aptr
           exit method.
       end method "compareTo".
      *>----------------------------------------------------------------
      *> Return True if the value of this CharacterArray is greater than
      *> the first compareLength bytes of compareValue, else return
      *> False.
      *>----------------------------------------------------------------
       method-id. "greaterThanByLengthValue".
       linkage section.
       01 comparevalue.
           03 abyte       pic x occurs 1 to maxnumberbytes depending on
                                   comparelength.
       01 lnkbool         pic x comp-5.
           88  lnkistrue  value 1.
           88  lnkisfalse value 0.
       01 comparelength        pic x(4) comp-5.

       procedure division using comparelength comparevalue
                                           returning lnkbool.
           if  instancedata(indexeddataoffset + 1:
                              storagerequirements) > comparevalue
               set lnkistrue to true
           else
               set lnkisfalse to true
           end-if
           exit method.
       end method "greaterThanByLengthValue".
      *>----------------------------------------------------------------
      *> Return true if the value of this CharacterArray is less than or
      *> equal to the value of anIntrinsic
      *>----------------------------------------------------------------
       method-id. "lessThanOrEqual".
       local-storage section.

       01 comparelength    pic x(4) comp-5.
       01 aptr             usage pointer.

       linkage section.
       01 comparevalue.
           03 abyte        pic x occurs 1 to maxnumberbytes depending on
                                   comparelength.
       01 anintrinsic      usage object reference.
       01 lnkbool          pic x comp-5.
           88  lnkistrue   value 1.
           88  lnkisfalse  value 0.

       procedure division using anintrinsic returning lnkbool.
           invoke anintrinsic "sizeInBytes" returning compareLength
           invoke self "malloc" using compareLength returning aptr
           set address of comparevalue to aptr
           invoke anintrinsic "getValue" returning compareValue
           if instancedata(indexeddataoffset + 1:
                           storagerequirements) <= comparevalue
               set lnkistrue to true
           else
               set lnkisfalse to true
           end-if
           invoke self "dealloc" using aptr
           exit method.
       end method "lessThanOrEqual".
      *>----------------------------------------------------------------
      *> Return true if the value of this CharacterArray is greater than
      *> or equal to the value of anIntrinsic
      *>----------------------------------------------------------------
       method-id. "greaterThanOrEqual".
       local-storage section.

       01 comparelength   pic x(4) comp-5.
       01 aptr            usage pointer.

       linkage section.
       01 comparevalue.
           03 abyte       pic x occurs 1 to maxnumberbytes depending on
                                   comparelength.
       01 anintrinsic     usage object reference.
       01 lnkbool         pic x comp-5.
           88  lnkistrue  value 1.
           88  lnkisfalse value 0.

       procedure division using anintrinsic returning lnkbool.
           invoke anintrinsic "sizeInBytes" returning compareLength
           invoke self "malloc" using compareLength returning aptr
           set address of comparevalue to aptr
           invoke anintrinsic "getValue" returning compareValue
           if instancedata(indexeddataoffset + 1:
                       storagerequirements) >= comparevalue
               set lnkistrue to true
           else
               set lnkisfalse to true
           end-if
           invoke self "dealloc" using aptr
           exit method.
       end method "greaterThanOrEqual".
      *>----------------------------------------------------------------
      *> Return True if the value indicated by compareValue and
      *> compareLength is Equal to the value of this CharacterArray
      *> else return False. However the comparison ignores the case of
      *> the characters. CompareLength is the number of bytes from
      *> compareValue to compare.
      *>----------------------------------------------------------------
       method-id. "equalsIgnoreCasebyLengthValue".
       local-storage section.

       01 temp            usage object reference.
       01 copyanobject    usage object reference.

       linkage section.
       01 comparevalue.
           03 abyte       pic x occurs 1 to maxnumberbytes depending on
                                   comparelength.
       01 comparelength   pic x(4) comp-5.
       01 lnkbool         pic x comp-5.
           88  lnkistrue  value 1.
           88  lnkisfalse value 0.

       procedure division using comparelength comparevalue
                  returning lnkbool.
           invoke selfClass "withLengthValue" using comparelength
                        compareValue
                                   returning copyanObject
           invoke self "equalsIgnoreCase" using copyanobject
                           returning lnkbool.
           invoke copyanobject "finalize" returning temp
           exit method.
       end method "equalsIgnoreCasebyLengthValue".
      *>----------------------------------------------------------------
      *> Return True if this CharacterArray is less than the
      *> first compareLength bytes of compareValue else return False.
      *>----------------------------------------------------------------
       method-id. "lessThanByLengthValue".
       linkage section.
       01 comparevalue.
           03 abyte       pic x occurs 1 to maxnumberbytes depending on
                                   comparelength.
       01 lnkbool         pic x comp-5.
           88  lnkistrue  value 1.
           88  lnkisfalse value 0.
       01 comparelength        pic x(4) comp-5.

       procedure division using comparelength comparevalue
                                         returning lnkbool.
           if  instancedata(indexeddataoffset + 1:
                              storagerequirements)
                               < comparevalue
               set lnkistrue to true
           else
               set lnkisfalse to true
           end-if
           exit method.
       end method "lessThanByLengthValue".
      *>----------------------------------------------------------------
      *> Set the value of this string to lnkString.
      *> Note that the number of bytes read from lnkstring is equal to
      *> the current size of the character array.
      *>----------------------------------------------------------------
       method-id. "setValue".
       01 dbcsString           usage object reference.
       01 numCharacters        pic x(4) comp-5.
       01 lsBool               pic x comp-5.
           88 containsDBCSCharacters           value 1.
           88 doesnotcontainDBCSCharacters     value 0.
       linkage section.
       01 lnkstring.
           03 abyte            pic x occurs 1 to maxnumberbytes
                               depending on storagerequirements.

       procedure division using lnkstring.
           if not isSBCSEnv
              invoke dbcsCharacterArray
                   "isDBCSStringLengthValue"
                       using storageRequirements
                             lnkString
                   returning lsBool
              if containsDBCSCharacters
                  invoke DBCSCHaracterArray "withLengthValue"
                         using storageRequirements
                               lnkString
                       returning dbcsString
                  invoke self "becomes" using dbcsString
                  invoke dbcsString "finalize" returning dbcsString
              else
                  move lnkstring to instancedata
                   (indexeddataoffset + 1 : storagerequirements)
              end-if
           else
              move lnkstring to instancedata
                   (indexeddataoffset + 1 : storagerequirements)
           end-if
           exit method.
       end method "setValue".
      *>----------------------------------------------------------------
      *> Returns the number of bytes required for this character array.
      *>----------------------------------------------------------------
       method-id. "sizeInBytes".
       linkage section.
       01 acollectionsize      pic s9(9) comp-5.

       procedure division returning acollectionsize.
           move numberelements to acollectionsize
           exit method.
       end method "sizeInBytes".
      *>----------------------------------------------------------------
      *> Concatenate this character array with the character array
      *> lnkobject returning the result in a new string.
      *>----------------------------------------------------------------
       method-id. "concat".
       local-storage section.
       01 j                    pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 aptr                 usage pointer.
       01 dbcsString           usage object reference.
       01 numCharacters        pic x(4) comp-5.
       01 lsBool               pic x comp-5.
           88 containsDBCSCharacters           value 1.
           88 doesnotcontainDBCSCharacters     value 0.

       linkage section.
       01 lnkworkingstring1.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on i.
       01 lnknewstring      usage object reference.
       01 lnkobject         usage object reference.
       01 lnkworkingstring2.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on j.

       procedure division using lnkobject returning lnknewstring.
           if not isSBCSEnv
               invoke lnkObject "containsDBCSCharacters"
                   returning lsBool
               if containsDBCSCharacters
                   invoke self "size" returning numCharacters
                   invoke dbcsCharacterArray "withLengthValue"
                                   using numCharacters
                               instanceData(indexedDataOffset + 1:)
                           returning dbcsString
                   invoke dbcsString "concat" using lnkObject
                                          returning lnknewstring
                   invoke dbcsString "finalize" returning dbcsString
               end-if
           else
               set doesnotcontainDBCSCharacters to true
           end-if
           if doesnotcontainDBCSCharacters
               invoke lnkobject "sizeInBytes" returning i
               move i to j
               add storagerequirements to i
               invoke self "malloc" using i returning aptr
               set address of lnkworkingstring1 to aptr
               move instancedata (indexeddataoffset + 1 :
                                       storagerequirements) to
                    lnkworkingstring1(1 : storagerequirements)
               invoke lnkObject "getValue" returning
                    lnkworkingstring1(storagerequirements + 1 : j)
               invoke selfClass "withLengthValue"
                           using i lnkWorkingString1
                       returning lnknewstring
               invoke self "dealloc" using aptr

           exit method.
       end method "concat".
      *>----------------------------------------------------------------
      *> Returns the value of this character array.
      *> Warning: If the buffer lnkstring is not large enough to hold
      *> the value in the characterarray then data corruption will
      *> occur. Invoking the method "sizeinbytes" will enable you to
      *> check if the buffer is large enough.
      *>----------------------------------------------------------------
       method-id. "getValue".
       linkage section.
       01 lnkstring.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on storagerequirements.

       procedure division returning lnkstring.
           move instancedata
                     (indexeddataoffset + 1 : storagerequirements)
               to lnkstring
           exit method.
       end method "getValue".

      *>----------------------------------------------------------------
      *> Returns the address of this characterArray as a cobol pointer.
      *>----------------------------------------------------------------
       method-id. "getAddress".
       linkage section.
       01 lnkPtr        POINTER.
       procedure division returning lnkPtr.
           set lnkPtr to address of instancedata
                     (indexeddataoffset + 1 : )
           exit method.
       end method "getAddress".

      *>----------------------------------------------------------------
      *> Returns the value of this character array as a parameter eg.
      *> null terminated
      *>----------------------------------------------------------------
       method-id. "asParameter".
       linkage section.
       01 lnkstring.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on storagerequirements.

       procedure division returning lnkstring.
           move instancedata
                     (indexeddataoffset + 1 : storagerequirements)
               to lnkstring
           move x"00" to lnkString(storageRequirements + 1: 1)
           exit method.
       end method "asParameter".

      *>----------------------------------------------------------------
      *> Assumes contents of this character array are numbers and
      *> returns the comp5 equivalent.
      *>----------------------------------------------------------------
       method-id. "asCOMP5".
       local-storage section.
       01 digit        pic 9.
       01 factor       pic x(4) comp-5.
       01 i            pic x(4) comp-5.
       linkage section.
       01 lnkComp5     pic s9(9) comp-5.
       procedure division returning lnkComp5.
           move 1 to factor
           move 0 to lnkComp5
           perform varying i from storageRequirements by -1 until i = 0
               move instancedata (indexeddataoffset + i :1) to digit
               compute lnkComp5 = lnkComp5 + (digit * factor)
               multiply 10 by factor
           end-perform
           exit method.
       end method "asCOMP5".

      *>----------------------------------------------------------------
      *> Display this character array
      *>----------------------------------------------------------------
       method-id. "display".
       procedure division.
           display instancedata
                 (indexeddataoffset + 1 : storagerequirements)
                 with no advancing
           exit method.
       end method "display".

      *>----------------------------------------------------------------
      *> Return a new string which contains the same characters as this
      *> string only in lower case
      *>----------------------------------------------------------------
       method-id. "asLowercase".
       local-storage section.
       01 aptr             usage pointer.
       linkage section.
       01 lnkworkingstring1.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on storageRequirements.
       01 lnknewstring     usage object reference.

       procedure division returning lnknewstring.
           invoke self "malloc" using storageRequirements
                            returning aptr
           set address of lnkworkingstring1 to aptr
           invoke self "getValue" returning lnkworkingstring1
           call "CBL_TOLOWER" using lnkworkingstring1
                               by value storageRequirements
           invoke selfClass "withlengthvalue" using storageRequirements
                                                 lnkworkingstring1
                                       returning lnknewstring
           invoke self "dealloc" using aptr
           exit method.
       end method "asLowercase".
      *>----------------------------------------------------------------
      *> Return an ordered collection of character arrays which
      *> represents each separate line of this character array.
      *>----------------------------------------------------------------
       method-id. "asSeparateLines".
       working-storage section.
       01 linefeed.
           03 intlinefeed      pic x comp-5 value 10.

       local-storage section.

       01 stringlength         pic s9(9) comp-5.
       01 startofline          pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 aline                usage object reference.

       linkage section.
       01 lnkcollection        usage object reference.

       procedure division returning lnkcollection.
           move 5 to i
           invoke orderedcollection "ofReferences" using i
                                               returning lnkcollection
           move 1 to startofline
           perform varying i  from 1 by 1 until i > storagerequirements
               if instancedata(indexeddataoffset + i : 1) = linefeed
                   compute stringlength = i - startofline
                   if stringlength = zero
                       continue
                   else
                       invoke characterarray "withLengthValue"
                               using stringlength
                                     instancedata(indexeddataoffset +
                                     startofline : stringlength)
                           returning aline
                       invoke lnkcollection "add"
                               using aline
                   end-if
                   move i to startofline
                   add 1 to startofline
               end-if
           end-perform
           if startofline <= storagerequirements
               if instancedata(indexeddataoffset + startofline : 1)
                       not = linefeed
                   compute stringlength = storagerequirements -
                                           startofline + 1
                   if stringlength = zero
                       continue
                   else
                       invoke characterarray "withLengthValue"
                               using stringlength
                                     instancedata(indexeddataoffset +
                                     startofline : stringlength)
                           returning aline
                            invoke lnkcollection "add"
                                   using aline
                   end-if
               end-if
           end-if
           exit method.
       end method "asSeparateLines".
      *>----------------------------------------------------------------
      *> Return a new string which contains the same characters as this
      *> string only in upper case
      *>----------------------------------------------------------------
       method-id. "asUppercase".
       local-storage section.
       01 aptr             usage pointer.
       linkage section.
       01 lnkworkingstring1.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on storageRequirements.
       01 lnknewstring     usage object reference.

       procedure division returning lnknewstring.
           invoke self "malloc" using storageRequirements
                            returning aptr
           set address of lnkworkingstring1 to aptr
           invoke self "getValue" returning lnkworkingstring1
           call "CBL_TOUPPER" using lnkworkingstring1
                               by value storageRequirements
           invoke selfClass "withlengthvalue" using storageRequirements
                                                 lnkworkingstring1
                                       returning lnknewstring
           invoke self "dealloc" using aptr
           exit method.
       end method "asUppercase".
      *>----------------------------------------------------------------
      *> Assumes contents of this character array are hex numbers and
      *> returns the comp5 equivalent.
      *> Note that this method only works for ascii systems
      *>----------------------------------------------------------------
       method-id. "asHEX".
       local-storage section.
       01 digit        pic x.
       01 digitByte    redefines digit pic x comp-5.
       01 tempByte     pic x comp-5.
       01 factor       pic x(4) comp-5.
       01 i            pic x(4) comp-5.
       linkage section.
       01 lnkComp5     pic 9(9) comp-5.
       procedure division returning lnkComp5.
           move 1 to factor
           move 0 to lnkComp5
           perform varying i from storageRequirements by -1 until i = 0
               move instancedata (indexeddataoffset + i :1) to digit
               if digit >= "A"
                  subtract 39 from digitByte
               end-if
               move 15 to tempByte
               call "CBL_AND" using tempByte
                                    digitByte
                              by value 1
               compute lnkComp5 = lnkComp5 + (digitByte * factor)
               multiply 16 by factor
           end-perform
           exit method.
       end method "asHEX".
      *>---------------------------------------------------------------
      *> Stores aValue at position anIndex. If anIndex is outside the
      *> bounds of this character array then an exception will be raised.
      *>---------------------------------------------------------------
       method-id. "atPut".
       local-storage section.
       01 realindex            pic s9(9) comp-5.
       01 charLength           pic x(4) comp-5.
       01 numCharacters        pic x(4) comp-5.
       01 aDBCSString          usage object reference.
       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 avalue.
           03 abyte            pic x occurs 0 to maxNumberBytes
                                     depending on collectionstride.
       procedure division using anindex avalue.
           if anindex < 1 or anindex > numberelements
               invoke self "errorinBounds"
           else
               if not isSBCSEnv
                   invoke DBCSCharacterArray "charLengthInBytes"
                                                       using aValue
                                                   returning charLength
                   if charLength = 1
                       compute realindex = (anindex - 1) + 1
                               + indexeddataoffset
                       move avalue to instancedata(realindex : 1)
                   else
                       invoke self "size" returning numCharacters
                       invoke DBCSCharacterArray "withLengthValue"
                             using numCharacters
                                   instancedata(indexedDataOffset + 1 :)
                           returning aDBCSString
                       invoke aDBCSString "atPut" using anIndex aValue
                       invoke aDBCSString "becomes" using self
                       invoke aDBCSString "finalize"
                           returning aDBCSString
                   end-if
               else
                   compute realindex = (anindex - 1) + 1
                           + indexeddataoffset
                   move avalue to instancedata(realindex : 1)
              end-if
           end-if
           exit method.
       end method "atPut".
      *>----------------------------------------------------------------
      *> Return an instance of cobolpicx which has the same value as
      *> this character array
      *>----------------------------------------------------------------
       method-id. "asPicX".

       local-storage section.

       01 aclass            usage object reference.

       linkage section.
       01 lnknewstring      usage object reference.

       procedure division returning lnknewstring.
           invoke cobolpicx "newClass" using storagerequirements
                                           returning aclass
           invoke aclass "withValue" using instancedata
                         (indexeddataoffset + 1 : storagerequirements)
                   returning lnknewstring
           exit method.
       end method "asPicX".
      *>----------------------------------------------------------------
      *> Return a new string stripped of all leading and trailing spaces
      *>----------------------------------------------------------------
       method-id. "trimBlanks".
       local-storage section.

       01 j                    pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 b                    pic s9(9) comp-5.

       linkage section.
       01 lnkobject            usage object reference.

       procedure division returning lnkobject.
           move 1 to b
           perform varying i from 1 by 1
             until instancedata(indexeddataoffset + i:b) not = space
                   or i > storagerequirements
           end-perform
           perform varying j from storagerequirements by -1
             until instancedata(indexeddataoffset + j:b) not = space
                   or j = 1
           end-perform
           add j to b
           subtract i from b
           if b < 0        *>>> Deals with the case where the
               move 0 to b *>>> whole string is spaces
           end-if
           invoke characterarray "withLengthValue" using b
                        instancedata(indexeddataoffset + i:b)
                returning lnkobject
           exit method.
       end method "trimBlanks".
      *>----------------------------------------------------------------
      *> Return True if this CharacterArray is less than the character
      *> array anArray else return False.
      *>----------------------------------------------------------------
       method-id. "lessThan".
       local-storage section.

       01 comparelength   pic x(4) comp-5.
       01 aptr            usage pointer.

       linkage section.
       01 comparevalue.
           03 abyte       pic x occurs 1 to maxnumberbytes depending on
                                   comparelength.
       01 anarray         usage object reference.
       01 lnkbool         pic x comp-5.
           88  lnkistrue  value 1.
           88  lnkisfalse value 0.

       procedure division using anarray returning lnkbool.
           invoke anarray "sizeInBytes" returning compareLength
           invoke self "malloc" using compareLength returning aptr
           set address of comparevalue to aptr
           invoke anarray "getValue" returning compareValue
           if instancedata(indexeddataoffset + 1:
                             storagerequirements)  < comparevalue
               set lnkistrue to true
           else
               set lnkisfalse to true
           end-if
           invoke self "dealloc" using aptr
           exit method.
       end method "lessThan".
      *>----------------------------------------------------------------
      *> Return True if this CharacterArray is greater than the
      *> character array anArray else return False.
      *>----------------------------------------------------------------
       method-id. "greaterThan".
       local-storage section.
       01 comparelength   pic x(4) comp-5.
       01 aptr            usage pointer.

       linkage section.
       01 comparevalue.
           03 abyte       pic x occurs 1 to maxnumberbytes depending on
                                   comparelength.
       01 anarray         usage object reference.
       01 lnkbool         pic x comp-5.
           88  lnkistrue  value 1.
           88  lnkisfalse value 0.

       procedure division using anarray returning  lnkbool.
           invoke anarray "sizeInBytes" returning compareLength
           invoke self "malloc" using compareLength returning aptr
           set address of comparevalue to aptr
           invoke anarray "getValue" returning compareValue
           if instancedata(indexeddataoffset + 1:
                             storagerequirements)  > comparevalue
               set lnkistrue to true
           else
               set lnkisfalse to true
           end-if
           invoke self "dealloc" using aptr
           exit method.
       end method "greaterThan".
      *>----------------------------------------------------------------
      *> Return a copy of this character array
      *>----------------------------------------------------------------
       method-id. "Copy".
       linkage section.
       01 result               usage object reference.

       procedure division returning result.
           invoke super "basicCopy" returning result
           exit method.
       end method "Copy".

      *>----------------------------------------------------------------
      *> Returns true if anotherstring is equal to this string otherwise
      *> false is returned.
      *>----------------------------------------------------------------
       method-id. "equal".
       local-storage section.

       01 anotherstringsize    pic x(4) comp-5.
       01 aptr                 usage pointer.

       linkage section.
       01 anotherstring        usage object reference.
       01 arealstring.
           03 achar            pic x occurs 1 to maxNumberBytes
                                    depending on anotherstringsize.
       01 lnkresult            pic x comp-5.
           88 isEqual          value 1.
           88 isNotEqual       value 0.

       procedure division using anotherstring returning lnkresult.
           invoke anotherString "sizeInBytes"
                    returning anotherstringsize
           invoke self "malloc" using anotherstringsize returning aptr
           set address of arealString to aptr
           invoke anotherstring "getValue" returning arealString
           if arealstring = instancedata
                (indexeddataoffset + 1 : storagerequirements)
               set isEqual to true
           else
               set isNotEqual to true
           end-if
           invoke self "dealloc" using aptr
           exit method.
       end method "equal".
      *>----------------------------------------------------------------
      *> Answer TRUE if the receiver is equal to anObject, else answer
      *> FALSE. The comparison is not case sensitive.
      *>----------------------------------------------------------------
       method-id. "equalsIgnoreCase".
       local-storage section.

       01 temp            usage object reference.
       01 copyself        usage object reference.
       01 copyanobject    usage object reference.

       linkage section.
       01 anobject        usage object reference.
       01 lnkbool         pic x comp-5.
           88  lnkistrue  value 1.
           88  lnkisfalse value 0.

       procedure division using anobject
                       returning lnkbool.
           invoke self "asUpperCase" returning copyself
           invoke anobject "asUpperCase" returning copyanobject
           invoke copyself "equal" using copyanobject
                                    returning lnkbool
           invoke copyself "finalize" returning temp
           invoke copyanobject "finalize" returning temp
           exit method.
       end method "equalsIgnoreCase".
      *>----------------------------------------------------------------
      *> Move the contents of lnkString to this String. if lnkString is
      *> longer than this string it is truncated. If it is shorter
      *> then this string is space filled.
      *>----------------------------------------------------------------

       method-id. "moveValue".
       local-storage section.
       01 i                    pic x(4) comp-5.
       01 aptr                 usage pointer.
       01 dbcsString           usage object reference.
       01 numCharacters        pic x(4) comp-5.
       01 lsBool               pic x comp-5.
           88 containsDBCSCharacters           value 1.
           88 doesnotcontainDBCSCharacters     value 0.
       linkage section.
       01 lnkString            usage object reference.
       01 lnkRealString.
           03 aChar pic x occurs 1 to maxnumberbytes depending on i.
       procedure division using lnkString.
           if not isSBCSEnv
               invoke lnkString "containsDBCSCharacters"
                           returning lsBool
               if containsDBCSCharacters
                   invoke self "size" returning numCharacters
                   invoke DBCSCharacterArray "withLengthValue"
                       using numCharacters
                             instanceData(indexedDataOffset + 1:)
                   returning dbcsString
                   invoke self "becomes" using dbcsString
                   invoke dbcsString "finalize" returning dbcsString
                   invoke self "moveValue" using lnkString
               end-if
           else
               set doesnotcontainDBCSCharacters to true
           end-if
           if doesnotcontainDBCSCharacters
               invoke lnkString "sizeInBytes" returning i
               invoke self "malloc" using i returning aptr
               set address of lnkRealString to aptr
               invoke lnkString "getValue" returning lnkRealString
               move lnkRealString to instanceData(indexedDataOffset + 1:
                                        storageRequirements)
               invoke self "dealloc" using aptr
           end-if
           exit method.
       end method "moveValue".
      *>----------------------------------------------------------------
      *> Reset the contents of this string so it has the same contents
      *> as lnkString.
      *>----------------------------------------------------------------

       method-id. "resetValue".
       local-storage section.
       01 dbcsString       usage object reference.

       linkage section.
       01 lnkString        usage object reference.
       procedure division using lnkString.
           invoke lnkString "copy" returning dbcsString
           invoke self "becomes" using dbcsString
           invoke dbcsString "finalize" returning dbcsString
           exit method.
       end method "resetValue".
      *>----------------------------------------------------------------
      *> Return a string containing a description of this string.
      *>----------------------------------------------------------------

       method-id. "description".
       local-storage section.
       01 lsSize       pic x(4) comp-5.
       01 lsPtr        usage pointer.
       linkage section.
       01 lnkString    usage object reference.
       01 lnkRealValue pic x.
       procedure division returning lnkString.
           invoke self "sizeInBytes" returning lsSize
           add 2 to lsSize
           invoke self "malloc" using lsSize returning lsPtr
           set address of lnkRealValue to lsPtr
           move """" to lnkRealValue(1:1)
           invoke self "getvalue" returning lnkRealValue(2 : lsSize)
           move """" to lnkRealValue(lsSize : 1)
           invoke selfClass "withLengthValue" using lsSize lnkRealValue
               returning lnkString
           invoke self "dealloc" using lsPtr
           exit method.
       end method "description".
      *>----------------------------------------------------------------
      *> Return the hash value of this Characterarray.
      *>----------------------------------------------------------------
       method-id. "hash".
       local-storage section.

       01 achar                    pic x.
       01 numchar redefines achar  pic x comp-5.
       01 y                        pic x(4) comp-5.
       01 j                        pic x(4) comp-5.
       01 i                        pic x(4) comp-5.

       linkage section.
       01 ahashvalue               pic s9(9) comp-5.

       procedure division returning ahashvalue.
           move zero to ahashvalue
           move 1 to j
           perform varying i from storagerequirements by -1
              until i = zero or
                    instancedata(indexeddataoffset + i:j) not  = space
           end-perform
           move i to y
           perform varying i from y by -1 until i = zero
               move instancedata(indexeddataoffset + i:j) to achar
               compute ahashvalue = ahashvalue * 47 + numchar
           end-perform
           if ahashvalue < 0
               compute ahashvalue = - ahashvalue
           end-if
           exit method.
       end method "hash".
      *>----------------------------------------------------------------
      *> Returns true if this collection and aCollection are compatible.
      *> Any instance method that takes a collection as a parameter must
      *> be compatible with the receiver of the message. An exception
      *> will be raised if the collections are incompatible.
      *>----------------------------------------------------------------
       method-id. "isCompatible".
       linkage section.
       01 lnkbool              pic x comp-5.
           88  lnkistrue       value 1.
           88  lnkisfalse      value 0.
       01 acollection          usage object reference.

       procedure division using acollection
                        returning lnkbool.
           invoke self "containsCharacters" returning lnkBool
           exit method.
       end method "isCompatible".
      *>----------------------------------------------------------------
      *> Return true if this type of collection contains characters
      *>----------------------------------------------------------------
       method-id. "containsCharacters".
       linkage section.
       01 lnkbool              pic x comp-5.
           88  lnkistrue       value 1.
           88  lnkisfalse      value 0.
       01 acollection          usage object reference.

       procedure division returning lnkbool.
           set lnkIsTrue to true
           exit method.
       end method "ContainsCharacters".
      *>----------------------------------------------------------------
      *> Causes this character array to become larger or smaller
      *> according to the requiredSize parameter.
      *>----------------------------------------------------------------
       method-id. "growTo".
       local-storage section.
       01 realindex            pic s9(9) comp-5.
       01 exceptionnumber      pic s9(9) comp-5.
       01 bool                 pic x comp-5.
          88 istrue            value 1.
          88 isfalse           value 0.
       01 oldsize              pic s9(9) comp-5.
       01 numbytestomove       pic s9(9) comp-5.
       01 newSize              pic s9(9) comp-5.
       linkage section.
       01 defaultreturn        usage object reference.
       01 requiredSize         pic x(4) comp-5.


       procedure division using requiredSize returning defaultreturn.
           invoke self "capacity" returning oldsize
           move requiredsize to numberelements
           move requiredsize to storagerequirements
           multiply collectionstride by storagerequirements
           add 1 to storageRequirements giving newSize
           invoke self "basicGrow" using newSize
                                    returning bool
           if isfalse
               move 32 to exceptionnumber
               invoke self "raiseException" using exceptionnumber
                                       returning defaultreturn
           else
               if oldsize < requiredsize then
                   compute realindex = (oldsize) * collectionstride + 1
                                   + indexeddataoffset
                   compute numbytestomove = storagerequirements + 1 +
                                        indexeddataoffset - realindex
                   if numBytesToMove > 0
                       move low-values to
                           instancedata(realindex:numbytestomove)
                   end-if
               else
                   compute realindex =
                           (storagerequirements * collectionstride)
                           + 1 + indexeddataoffset
                   move low-values to instancedata(realindex:1)
               end-if
               set defaultreturn to self
           end-if
           exit method.
       end method "growTo".
      *>----------------------------------------------------------------
      *> Returns the first lnksize bytes of this characterarray
      *>----------------------------------------------------------------
       method-id. "getValueWithSize".
       linkage section.
       01 lnksize          pic 9(9) comp-5.
       01 lnkstring.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on storagerequirements.


       procedure division using lnksize returning lnkstring.
           if storagerequirements < lnksize
             move instancedata
                       (indexeddataoffset + 1 : storagerequirements)
               to lnkstring(1 : lnksize)
           else
             move instancedata
                       (indexeddataoffset + 1 : lnksize)
               to lnkstring(1 : lnksize)
           end-if
           exit method.
       end method "getValueWithSize".

      *>----------------------------------------------------------------
      *> Returns the lnksize bytes of this characterarray starting
      *> at position lnkoffset.
      *>----------------------------------------------------------------
       method-id. "getValueWithSizeAtPosition".
       linkage section.
       01 lnksize          pic 9(9) comp-5.
       01 lnkoffset        pic 9(9) comp-5.
       01 lnkstring.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on storagerequirements.

       procedure division using lnkoffset lnksize returning lnkstring.
           if storagerequirements < lnksize + lnkoffset - 1
             move instancedata
                   (indexeddataoffset + lnkoffset : storagerequirements)
               to lnkstring(1 : lnksize)
           else
             move instancedata
                   (indexeddataoffset + lnkoffset : lnksize)
               to lnkstring(1 : lnksize)
           end-if
           exit method.
       end method "getValueWithSizeAtPosition".

      *>----------------------------------------------------------------
      *> Delimits the character array by white space and places each
      *> delimited string element into an Array of references.
      *> White space in this method, is defined as being any character
      *> with an ascii value that is less than or equal to 32 (space).
      *>----------------------------------------------------------------
       method-id. "asArrayOfSubStrings".

       local-storage section.
       01 lsByteSize                   pic s9(9) comp-5.
       01 lsCharSize                   pic s9(9) comp-5.
       01 lsPtr                        usage pointer.
       01 lsCollectionSize             pic s9(9) comp-5.
       01 lsOrderedCollectionObject    usage object reference.
       01 lsSearchStatus               pic x comp-5.
          88 lsSearchForNonNoiseChar       value 0.
          88 lsSearchForNoiseChar          value 1.
       01 lsWordStart                  pic s9(9) comp-5.
       01 lsWordCharLength             pic s9(9) comp-5.
       01 lsCharIndex                  pic s9(9) comp-5.
       01 lsWordObject                 usage object reference.

       linkage section.
       01 lnkArrayObject               usage object reference.
       01 lnkRealString.
          03 lnkRealChar               pic x occurs 0 to maxNumberBytes
                                         depending on lsByteSize.

       procedure division returning lnkArrayObject.
           invoke self "sizeInBytes"
                       returning lsByteSize
           invoke self "malloc"
                           using lsByteSize
                       returning lsPtr
           set address of lnkRealString to lsPtr
           invoke self "getValue"
                       returning lnkRealString
           move 10 to lsCollectionSize
           invoke OrderedCollection "ofReferences"
                           using lsCollectionSize
                       returning lsOrderedCollectionObject
           invoke self "size"
                       returning lsCharSize
           set lsSearchForNonNoiseChar to true
           move zero to lsWordCharLength
           perform varying lsCharIndex from 1 by 1
             until lsCharIndex > lsCharSize
               if lsSearchForNonNoiseChar
                   if lnkRealString(lsCharIndex:1) <= space
                       continue
                   else
                       move lsCharIndex to lsWordStart
                       add 1 to lsWordCharLength
                       set lsSearchForNoiseChar to true
                   end-if
               else
                   if lnkRealString(lsCharIndex:1) <= space
                       set lsSearchForNonNoiseChar to true
                       invoke characterArray "withLengthValue"
                                       using lsWordCharLength
                                             lnkRealString(lsWordStart:
                                               lsWordCharLength)
                                   returning lsWordObject
                       invoke lsOrderedCollectionObject "add"
                                       using lsWordObject
                       move zero to lsWordCharLength
                   else
                       add 1 to lsWordCharLength
                   end-if
               end-if
           end-perform
           if lsWordCharLength = zero
               continue
           else
               invoke characterArray "withLengthValue"
                               using lsWordCharLength
                                     lnkRealString(lsWordStart:
                                       lsWordCharLength)
                           returning lsWordObject
               invoke lsOrderedCollectionObject "add"
                               using lsWordObject
           end-if
           invoke lsOrderedCollectionObject "asArray"
                       returning lnkArrayObject
           invoke lsOrderedCollectionObject "finalize"
                       returning lsOrderedCollectionObject
           invoke self "dealloc"
                           using lsPtr
           exit method.
       end method "asArrayOfSubStrings".
      *>----------------------------------------------------------------
      *> Search through this Character Array starting at position
      *> lnkStartCharIndexPosition looking for the first occurance
      *> of the contents of lnkSearchCharacterArray.
      *> If the search is sucessful, then the index of
      *> lnkSearchCharacterArray within this Character Array will be
      *> returned in lnkCharIndexPosition, otherwise zero will be
      *> returned.
      *>----------------------------------------------------------------
       method-id. "indexOfSubCollection".

       local-storage section.
       01 aPtr                         usage pointer.
       01 exceptionNumber              pic s9(9) comp-5.
       01 searchCharSize               pic s9(9) comp-5.
       01 searchByteSize               pic s9(9) comp-5.
       01 indexOffset                  pic x(4) comp-5.
       01 charIndexPosition            pic x(4) comp-5.
       01 aBool                        pic x comp-5.
          88 isFalse                       value 0.
          88 isTrue                        value 1.

       linkage section.
       01 lnkSearchCharacterArray      usage object reference.
       01 lnkStartCharIndexPosition    pic s9(9) comp-5.
       01 lnkCharIndexPosition         pic s9(9) comp-5.

       01 lnkSearchString.
          03 lnkSearchByte             pic x  occurs 0 to maxNumberBytes
                                           depending on searchByteSize.


       procedure division using lnkSearchCharacterArray
                                lnkStartCharIndexPosition
                      returning lnkCharIndexPosition.

           invoke self "isCompatible"
                           using lnkSearchCharacterArray
                       returning aBool
           if isTrue
               move zero to lnkCharIndexPosition
               invoke lnkSearchCharacterArray "size"
                           returning searchCharSize
               if searchCharSize = zero
                   move 11 to exceptionnumber
                   invoke self "raiseException"
                                   using exceptionNumber
                               returning lnkCharIndexPosition
               else
                   invoke lnkSearchCharacterArray "sizeInBytes"
                               returning searchByteSize
                   invoke self "malloc"
                                   using searchByteSize
                               returning aPtr
                   set address of lnkSearchString to aPtr
                   invoke lnkSearchCharacterArray "getValue"
                               returning lnkSearchString
                   add lnkStartCharIndexPosition to indexedDataOffset
                                     giving indexOffset
                   perform varying charIndexPosition
                     from lnkStartCharIndexPosition by 1
                     until charIndexPosition > numberElements or
                           charIndexPosition + searchCharSize - 1
                           > numberElements or
                           lnkCharIndexPosition not = zero
                       if instanceData(indexOffset:searchByteSize) =
                          lnkSearchString
                           move charIndexPosition to
                                lnkCharIndexPosition
                       else
                           add 1 to indexOffset
                       end-if
                   end-perform
                   invoke self "dealloc" using aPtr
               end-if
           else
               invoke self "compatibilityError"
                           returning lnkCharIndexPosition
           end-if
           exit method.
       end method "indexOfSubCollection".
      *>----------------------------------------------------------------
      *> Return a new character array that will contain exactly the
      *> same characters between the specified indices of this character
      *> array.
      *>----------------------------------------------------------------
       method-id. "copyFromTo".
       local-storage section.
       01 lsCharSize           pic s9(9) comp-5.
       01 lsOffset             pic s9(9) comp-5.
       01 lsExceptionNumber    pic s9(9) comp-5.

       linkage section.
       01 lnkStartIndex        pic s9(9) comp-5.
       01 lnkEndIndex          pic s9(9) comp-5.
       01 lnkNewCharacterArray usage object reference.

       01 lnkRealString.
           03 lnkByte           pic x occurs 0 to maxnumberbytes
                                   depending on lsCharSize.

       procedure division using lnkStartIndex
                                lnkEndIndex
                      returning lnkNewCharacterArray.

           subtract lnkStartIndex from lnkEndIndex
                                giving lsCharSize
           add 1 to lsCharSize
           if lsCharSize < 1
               invoke self "errorInBounds"
                           returning lnkNewCharacterArray
           else
               add indexedDataOffset to lnkStartIndex
                               giving lsOffset
               invoke characterArray "withLengthValue"
                               using lsCharSize
                                     instanceData(lsOffset:lsCharSize)
                           returning lnkNewCharacterArray
           end-if
           exit method.
       end method "copyFromTo".

      *>----------------------------------------------------------------
      *> Returns the value of this character array as null terminated.
      *> Warning: If the buffer lnkstring is not large enough to hold
      *> the value in the characterarray plus one byte for the null,
      *> then data corruption will occur. Invoking the method
      *> "sizeinbytes" will enable you to check if the buffer is
      *> large enough.
      *>----------------------------------------------------------------
       method-id. "getValueZ".
       linkage section.
       01 lnkstring.
           03 abyte        pic x occurs 1 to maxnumberbytes
                           depending on storagerequirements.

       procedure division returning lnkstring.
           move instancedata
                     (indexeddataoffset + 1 : storagerequirements)
               to lnkstring
           move x"00" to lnkString(storageRequirements + 1: 1)
           exit method.
       end method "getValueZ".

       end object.
       end class characterarray.
