      *>----------------------------------------------------------------
      *> A CallBack stores a message which may be Invoked later. Send the
      *> message "invoke" to a CallBack to invoke the message stored.
      *> Note the maximum size of a message is 30 characters.
      *>
      *> Copyright (c) Micro Focus Ltd 1991, 1996
      *>
      *> Some edits
      *>----------------------------------------------------------------
      $set  nosmalldd mfoo
       class-id. entryCallback data is protected
               inherits from Dependent
                  with data.

        object section.
        class-control.
           CharacterArray is class "chararry"
           entryCallback is class "entrycll"
           Dependent is class "dependnt".

       working-storage section.

       object-storage section.
      *>----------------------------------------------------------------
      *> Return an instance of EntryCallback with the entry name
      *> aMessageName and p1 - p6 extra parameters.
      *> The addresses of the parameters are stored and passed
      *> preceeding the actual parameters when this EntryCallBack is 
      *> Invoked. If more than six extra parameters are specified
      *> then an exception is raised.
      *>----------------------------------------------------------------
       method-id. "new".
       local-storage section.
       01  local-stuff.
        02 exceptionnumber          pic s9(9) comp-5.
        02 sendObjectHandle         usage object reference.
        02 messageText              pic x(80).
        02 parmPointer              usage pointer.
        02 serverid                 pic x(4) comp-5.

       linkage section.
       01 p1                   usage object reference.
       01 p2                   usage object reference.
       01 p3                   usage object reference.
       01 p4                   usage object reference.
       01 p5                   usage object reference.
       01 p6                   usage object reference.
       01 p7notallowed         usage object reference.
       01 result               usage object reference.
       01 amessagename         pic x(80).
       01 parmDescriptor       pic x.

       procedure division using amessagename p1 p2 p3 p4 p5 p6
                                       p7notallowed
                                       returning result.
           if address of p7notallowed not = null
               move 20 to exceptionnumber
               invoke self "raiseException" using exceptionnumber
                                             returning result
               exit method
           else
               invoke super "basicnew" returning result
               invoke result "initialize" using amessagename
                                                   p1
                                                   p2
                                                   p3
                                                   p4
                                                   p5
                                                   p6

           end-if
           exit method.
       end method "new".
       Object.

       object-storage section.

       01 finalizeStatus       pic x comp-5.
                88 requestFinalize value 1.

      *>>---------------------------------------------------------------
      *>> The name of the method to be isInvoked
      *>>---------------------------------------------------------------

       01 messagename          pic x(81). *>> 80 + 1 for the space
                                          *>>terminator

       78 maxnumparams         value 9.
      *>>---------------------------------------------------------------
      *>> A cobol table storing the addresses of the parameters passed
      *>> when an instance of BoundMethod is created
      *>>---------------------------------------------------------------

       01 parameters occurs maxnumparams.
          05 Objectparms       usage object reference.

      *>>---------------------------------------------------------------
      *>> The number of parameters passed at instantiation
      *>>---------------------------------------------------------------
       01 numparams            pic x(4) comp-5.


      *>>---------------------------------------------------------------
      *>> Counts how many times this bound method has been isInvoked in
      *>> the current call chain
      *>>---------------------------------------------------------------

       01 isInvoked            pic x(4) comp-5.

      *>----------------------------------------------------------------
      *> Return true if this bound method is executing in the current
      *> call chain
      *>----------------------------------------------------------------
       method-id. "active".
       linkage section.
       01 bool             pic x comp-5.
           88 isfalse      value 0.
           88 istrue       value 1.

       procedure division returning bool.
           if isInvoked > 0
               set istrue to true
           else
               set isfalse to true
           end-if
           exit method.
       end method "active".
      *>----------------------------------------------------------------
      *> Return the message of the Callback. Returns 81 characters as
      *> it includes the space terminator
      *>----------------------------------------------------------------
       method-id. "getMessage".
       linkage section.
       01 lnkMessage        pic x(81).
       procedure division returning lnkMessage.
           move messageName to lnkMessage
           exit method.
       end method "getMessage".

      *>----------------------------------------------------------------
      *> Return a description for this instance.
      *>----------------------------------------------------------------
       method-id. "description".
       01 wsString          pic x(160) value " for code entitled ".
       local-storage section.
       01 lsString         usage object reference.
       01 lsString2        usage object reference.       
       01 i                pic x(4) comp-5.

       linkage section.
       01 lnkMessage        pic x(81).
       procedure division returning lnkMessage.
           invoke super "description" returning lsString
           move messageName to wsString(20:)
           perform varying i from 160 by -1 
                        until wsString(i:1) not = space
           end-perform
           invoke CharacterArray "withLengthValue" using i wsString
                                        returning lsString2
           invoke lsString "concat" using lsString2 
                                returning lnkMessage
           invoke lsString "finalize" returning lsString           
           invoke lsString2 "finalize" returning lsString2
           exit method.
       end method "description".

      *>----------------------------------------------------------------
      *> Returns true if self's hashValue = lnkCallback's hashValue
      *>----------------------------------------------------------------
       method-id. "equal".
       local-storage section.
       01 lsMessage          pic x(81).
       01 lsClass            object reference.

       linkage section.
       01 lnkCallback        object reference.
       01 lnkBool            pic x comp-5.
          88 isTrue          value 1.
          88 isFalse         value 0.

       procedure division using lnkCallback returning lnkBool.
           invoke lnkCallback "getMessage" returning lsMessage
           invoke lnkCallback "class" returning lsClass
           if messageName = lsMessage and lsClass = selfclass
               set isTrue to true
           else
               set isFalse to true
           end-if
           exit method.
       end method "equal".
      *>----------------------------------------------------------------
      *> Return the value of the nth parameter
      *>----------------------------------------------------------------
       method-id. "getParameter".
       local-storage section.
       01 exceptionNumber  pic x(4) comp-5.
       linkage section.
       01 n                pic x(4) comp-5.
       01 anObject         usage object reference.
       procedure division using n returning anObject.
           if n <= numParams
               set anObject to objectParms(n)
           else
               move 17 to exceptionNumber
               invoke self "raiseException" using exceptionNumber
                           returning anObject
           end-if
           exit method.
       end method "getParameter".
      *>----------------------------------------------------------------
      *> Destroy this callback if it is not on the stack.
      *>----------------------------------------------------------------
       method-id. "finalize".
       linkage section.
       01 anObject         usage object reference.
       procedure division returning anObject.
           if isInvoked = zero
               invoke super "finalize" returning anObject
           else
               set requestFinalize to true
           end-if
           exit method.
       end method "finalize".

      *>----------------------------------------------------------------
      *> PRIVATE. Initialize the new instance of this entryCallback.
      *>----------------------------------------------------------------

       method-id. "initialize".
       local-storage section.
       01 i pic x(4) comp-5.
       01 aString  usage object reference.

       linkage section.
       01 p1                   usage object reference.
       01 p2                   usage object reference.
       01 p3                   usage object reference.
       01 p4                   usage object reference.
       01 p5                   usage object reference.
       01 p6                   usage object reference.
       01 amessagename         pic x(80).

       procedure division using amessagename p1 p2
                                                       p3 p4 p5 p6.
           move zero to numParams
           if address of p1 not = null
               set objectParms(1) to p1
               move 1 to numParams
               if address of p2 not = null
                   set objectParms(2) to p2
                   move 2 to numParams
                   if address of p3 not = null
                       set objectParms(3) to p3
                       move 3 to numParams
                       if address of p4 not = null
                           set objectParms(4) to p4
                           move 4 to numParams
                           if address of p5 not = null
                               set objectParms(5) to p5
                               move 5 to numParams
                               if address of p6 not = null
                                   set objectParms(6) to p6
                                   move 6 to numParams
                               end-if
                           end-if
                       end-if
                   end-If
               end-if
           end-if
           move spaces to messagename
           invoke characterArray "withSpaceOrNullTerminator"
                       using amessageName
                   returning aString
           invoke aString "sizeInBytes" returning i
           if i > 79
               move "entryCallbackMethodNameTooLong" to messageName
           else
               invoke aString "getValue" returning messageName
           end-if
           invoke aString "finalize" returning aString
           move zero to isInvoked
           exit method.
       end method "initialize".
      *>----------------------------------------------------------------
      *> Determine how many parameters there are and call the method
      *> with that number of paramters. The number of parameters is
      *> determined by calculating the sum of the optional parameters
      *> passsed when this CallBack was instantiated and the number of
      *> parameters passed on the invoke message. The parameters passed
      *> on this call succeed those passed at instantiation. The maximum
      *> number of parameters that can be passed with the invoke call is
      *> three, if any more are passed then an exception will be raised.
      *> Return the result of the method executed.
      *>----------------------------------------------------------------

       method-id. "invoke".
       local-storage section.
       01  local-stuff.
        02 ooreturnptr              usage object reference.
        02 exceptionnumber          pic s9(9) comp-5.
        02 numparamsplusargs        pic s9(9) comp-5.
        02 i                        pic x(4) comp-5.
       01 lsObject                  usage object reference.

       01 theParameters occurs maxnumparams.
          05 parm              usage pointer.

       linkage section.
       01 p1                   usage object reference.
       01 p2                   usage object reference.
       01 p3                   usage object reference.
       01 p4                   usage object reference.
       01 p5                   usage object reference.
       01 p6                   usage object reference.
       01 p7                   usage object reference.
       01 p8                   usage object reference.
       01 p9                   usage object reference.
       01 blockeval            usage object reference.
       01 anarg1               usage object reference.
       01 anarg2               usage object reference.
       01 anarg3               usage object reference.
       01 anarg4               usage object reference.

       procedure division using anarg1 anarg2 anarg3 anarg4
                            returning blockeval.
           if address of blockeval = null
               set address of blockeval to address of ooreturnptr
           end-if
           if address of anarg4 not = null
               move 21 to exceptionnumber
               invoke self "raiseException" using exceptionnumber
                                             returning blockeval
           else
               perform varying i from 1 by 1 until i > numParams
                   set parm(i) to address of objectParms(i)
               end-perform
               move numparams to numparamsplusargs
               if address of anarg1 not = null
                   add 1 to numparamsplusargs
                   set parm(numparamsplusargs) to address of anarg1
                   if address of anarg2 not = null
                       add 1 to numparamsplusargs
                       set parm(numparamsplusargs) to address of anarg2
                       if address of anarg3 not = null
                           add 1 to numparamsplusargs
                           set parm(numparamsplusargs) to
                                                      address of anarg3
                       end-if
                   end-if
               end-if
               set address of p1 to parm(1)
               set address of p2 to parm(2)
               set address of p3 to parm(3)
               set address of p4 to parm(4)
               set address of p5 to parm(5)
               set address of p6 to parm(6)
               set address of p7 to parm(7)
               set address of p8 to parm(8)
               set address of p9 to parm(9)
               add 1 to isInvoked
               evaluate numparamsplusargs
               when 0
               call messagename returning blockeval
               when 1
               call messagename using p1 returning blockeval
               when 2
               call messagename using p1 p2 returning blockeval
               when 3
               call messagename using p1 p2 p3 returning blockeval
               when 4
               call messagename using p1 p2 p3 p4
                                               returning blockeval
               when 5
               call messagename using p1 p2 p3 p4 p5
                                               returning blockeval
               when 6
               call messagename using p1 p2 p3 p4 p5 p6
                                               returning blockeval
               when 7
               call messagename using p1 p2 p3 p4 p5 p6 p7
                                               returning blockeval
               when 8
               call messagename using p1 p2 p3 p4 p5 p6 p7 p8
                                               returning blockeval
               when 9
               call messagename using p1 p2 p3 p4 p5 p6 p7 p8 p9
                                               returning blockeval
               end-evaluate
               subtract 1 from isInvoked
               if requestFinalize and isInvoked = 0
                        invoke self "finalize" returning lsObject
               end-if
           end-if
           exit method.
       end method "invoke".
       end object.
       end class entryCallback.
