      
      *>----------------------------------------------------------------
      *> A SortedCollection is similar to an OrderedCollection; however
      *> elements are stored in an order defined by a sort method. The
      *> default order is ascending.
      *>
      *> Copyright (C) 1991,1996 Micro Focus Ltd.
      *>
      *>----------------------------------------------------------------
      $set linkcount(150) mfoo
       class-id. sortedcollection data is protected
                     inherits from dynamicArrayedCollection with data.


        object section.
        class-control.
           dynamicArrayedCollection is class "dynarray"
           sortedcollection is class "srtdclln"
           Orderedcollection is class "ordrdcll"
           CharacterArray is class "chararry"
           callback is class "callback"
           .

       object-storage section.

      *>----------------------------------------------------------------
      *> Return an instance of a SortedCollection containing the
      *> elements of aCollection in an order defined by the default sort
      *> method (lnkreceiver / lnkSelector)
      *>----------------------------------------------------------------
       method-id. "withAllandSortMethod".
       local-storage section.

       01 atemplate            usage object reference.
       01 asize                pic s9(9) comp-5.

       linkage section.
       01 anewcollection       usage object reference.
       01 lnkselector          pic x(80).
       01 lnkreceiver          usage object reference.
       01 acollection          usage object reference.

       procedure division using acollection lnkreceiver
                                                lnkselector
                          returning anewcollection.
           invoke acollection "size" returning asize
           invoke acollection "template" returning atemplate
           invoke self "ofValues" using atemplate asize
                                       returning anewcollection
           invoke anewcollection "setSortMethod" using lnkreceiver
                                                       lnkselector
           invoke anewcollection "addAll" using acollection
           exit method.
       end method "withAllandSortMethod".

       Object.

       object-storage section.


      *>>---------------------------------------------------------------
      *>> The implementor of the method.
      *>>---------------------------------------------------------------
       01 methodimplementor      usage object reference value null.
       01 methodName             pic x(80).
       01 cacheBlock             pic x(4).
       01 hashValue              pic x(4) comp-5.

      *>----------------------------------------------------------------
      *> Create a CallBack based on the parameters lnkReceiver and
      *> lnkSelector to represent the method used to determine
      *> the sort order of the elements.
      *>----------------------------------------------------------------
       method-id. "setSortMethod".
       01 i pic x(4) comp-5.
       01 j                    pic x(4) comp-5.
       01 aMask                pic x(4) comp-5.
       01 aString              usage object reference.                        
       01 hashDigit            pic x.
       01 hashDigitAsNum redefines hashDigit pic x comp-5.

       linkage section.
       01 lnkselector          pic x(80).
       01 lnkreceiver          usage object reference.

       procedure division using lnkreceiver lnkselector.
           move spaces to methodName
           invoke characterArray "withSpaceOrNullTerminator"
                       using lnkSelector
                   returning aString
           invoke aString "sizeInBytes" returning i
           if i > 79
               move "callbackmethodnametoolong" to methodName
           else
               invoke aString "getValue" returning methodName
               call "CBL_TOLOWER" using methodName by value i
           end-if
           invoke aString "finalize" returning aString
           set methodimplementor to lnkreceiver
           move low-values to cacheBlock
           move zero to hashValue
           perform varying j from 1 by 1 until j > i
               multiply 2 by hashValue
               move methodName(j:1) to hashDigit
               add hashDigitAsNum to hashValue
           end-perform
           move h"FFFF" to aMask
           call "CBL_AND" using aMask
                                hashValue
                                by value 4
           invoke self "resort"
           exit method.
       end method "setSortMethod".
      *>----------------------------------------------------------------
      *> PRIVATE. Used to initialize the instance variables for new
      *> instances of SortedCollection.
      *>----------------------------------------------------------------
       method-id. "initialize".
       local-storage section.
       01 anObject             usage object reference.
       01 messageSelector      pic x(80).
       linkage section.
       01 lnktemplate          usage object reference.
       01 lnksize              pic s9(9) comp-5.
       01 lnkstride             pic s9(9) comp-5.

       procedure division using lnksize lnktemplate lnkstride.
           invoke super "initialize" using lnksize lnktemplate lnkstride
           move "defaultSortMethod " to messageselector           
           invoke self "setSortMethod" using self messageSelector
           exit method.
       end method "initialize".

      *>----------------------------------------------------------------
      *> Store anElement within this SortedCollection. The position
      *> where this is stored depends on anElement's relation to the
      *> other elements and the sort method.
      *>----------------------------------------------------------------
       method-id. "add".
       local-storage section.
       01 i                    pic s9(9) comp-5.

       linkage section.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anelement.
           invoke self "indexForInserting" using anelement
                                            returning i
           invoke super "insertBeforeIndex" using anelement i
           exit method.
       end method "add".
      *>----------------------------------------------------------------
      *> Add all the elements of aCollection to this collection, and
      *> then resort.
      *>----------------------------------------------------------------
       method-id. "addAll".
       local-storage section.

       01 acallback         usage object reference.
       01 messageselector   pic x(80).
       01 bool              pic x comp-5.
           88 istrue           value 1.
           88 isfalse          value 0.

       linkage section.
       01 acollection          usage object reference.
       procedure division using acollection.
           invoke self "isCompatible" using acollection
                                       returning bool
           if istrue
               move "addAllBM" to messageselector
               invoke callback "new" using self messageselector
                                 returning acallback
               invoke acollection "do" using acallback
               invoke self "resort"
           else
               invoke self "compatibilityError"
           end-if
           exit method.
       end method "addAll".
      *>----------------------------------------------------------------
      *> PRIVATE. CallBack Used to implement addAll
      *>----------------------------------------------------------------
       method-id. "addAllBM".
       linkage section.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       procedure division using anelement.
           invoke super "add" using anelement
           exit method.
       end method "addAllBM".
      *>----------------------------------------------------------------
      *> Raise an exception as SortedCollections should not implement
      *> this method.
      *>----------------------------------------------------------------
       method-id. "atPut".
       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anindex anelement.
           invoke self "shouldNotImplement"
           exit method.
       end method "atPut".
      *>----------------------------------------------------------------
      *> PRIVATE. Sorts the whole collection according to the sortBlock
      *>----------------------------------------------------------------
       method-id. "reSort".
       local-storage section.
       01 j                    pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 aptr                 pointer.
       01 aptr2                pointer.
       01 aptr3                pointer.

       procedure division.
           move collectionStride to i
           multiply 2 by i
           invoke self "malloc" using i returning aPtr
           set aptr2 to aptr
           set aptr2 up by collectionStride

           compute i = (firstindex - 1) * collectionstride + 1 +
                       indexeddataoffset
           compute j = (lastindex - 1) * collectionstride + 1 +
                       indexeddataoffset
           
           invoke self "sort" using i j
                                   by value aPtr
                                   by value aPtr2

           invoke self "dealloc" using aptr
           exit method.
       end method "reSort".
      *>---------------------------------------------------------------
      *> PRIVATE. Sorts the whole collection according to
      *> the sortMethod.
      *>---------------------------------------------------------------
       method-id. "sort".
       local-storage section.
       01 procPtr              usage procedure-pointer.
       01 mid                  pic s9(9)  comp-5.
       01 temp                 pic s9(9)  comp-5.
       01 k                    pic s9(9)  comp-5.
       01 j                    pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 n                    pic s9(9) comp-5.
       01 bool                 pic x comp-5.
           88 istrue            value 1.
           88 isfalse           value 0.
       01 sortStatus           pic x comp-5.
           88 finishedSort      value 1.
           88 notfinishedSort   value 0.

       linkage section.
       01 anelement2.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 anelement3.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       01 toindex              pic s9(9) comp-5.
       01 fromindex            pic s9(9) comp-5.

       procedure division using fromindex toindex
                                   anelement2 anElement3.
           move fromindex to i
           move toindex to j
           set notfinishedSort to true
        perform with test after until finishedSort
           subtract i from j giving n
           if n <= collectionStride
               set finishedSort to true
               if n = collectionStride
                   move  instancedata(j:collectionstride) to anelement2
                   call "oopsresolve"
                          using by reference methodName
                                by reference methodImplementor
                                by value hashValue
                                by reference cacheblock
                        returning procPtr
                   call procptr using bool
                                       anelement2
                                       instancedata(i:collectionstride)
                   if isTrue
                       move instancedata(i:collectionstride) to
                                anElement3
                       move instancedata(j:collectionstride) to
                                instancedata(i:collectionstride)
                       move anElement3 to
                                instancedata(j:collectionstride)
                   end-if
               end-if
           else
               move i to mid
               add j to mid
               subtract indexedDataOffset from mid
               subtract indexedDataOffset from mid
               subtract 2 from mid
               divide 2 into mid
               divide collectionStride into mid

               *>>Compute mid = ((i - indexedDataOffSet - 1) +
               *>>             (j - indexedDataOffset - 1)) /
               *>>             (collectionStride * 2)
               multiply collectionStride by mid

               add indexedDataOffset to mid
               add 1 to mid

               move instancedata(mid:collectionstride) to
                                         anelement2
               move instancedata(i:collectionstride) to
                    instancedata(mid:collectionstride)
               move i to n, k
               perform with test after until n >= j
                   add collectionStride to n
                   call "oopsresolve" 
                          using by reference methodName
                                by reference methodImplementor
                                by value hashValue
                                by reference cacheblock
                        returning procPtr
                   call procptr using bool
                                   instancedata(n:collectionstride)
                                   anelement2

                   if isTrue
                       add collectionStride to k
                       if k not = n
                           move instancedata(n:collectionstride) to
                                anElement3
                           move instancedata(k:collectionstride) to
                                instancedata(n:collectionstride)
                           move anElement3 to 
                                instancedata(k:collectionstride)

                       end-if
                   end-if
               end-perform
               move instancedata(k:collectionstride) to
                   instancedata(i:collectionstride)
               move anElement2 to instancedata(k:collectionstride)
               if mid < k
                   add collectionStride to k giving temp
                   invoke self "sort" using temp j
                                            anElement2
                                            anElement3
                   move k to j
                   subtract collectionStride from j
               else
                   subtract collectionStride from k giving temp
                   invoke self "sort" using i temp
                                            anElement2
                                            anElement3
                   move k to i
                   add collectionStride to i
               end-if
           end-if
       end-perform
       exit method.


       end method "sort".
      *>----------------------------------------------------------------
      *> PRIVATE. Returns an index which represents the index position
      *> before which anElement should be inserted.
      *>----------------------------------------------------------------
       method-id. "indexForInserting".
       local-storage section.
       01 procPtr              usage procedure-pointer.
       01 anobject            usage object reference.
       01 realindex           pic s9(9) comp-5.
       01 bool                pic x comp-5.
           88 istrue           value 1.
           88 isfalse          value 0.
       01 aptr                 pointer.
       01 k                    pic s9(9)  comp-5.
       01 j                    pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 aBuffer              pic x(20).

       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 anelement2.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anelement returning anindex.
           move firstindex to i
           move lastindex to j
           compute k = (i + j) / 2
           if collectionStride <= 20
               set address of anElement2 to address of aBuffer
           else
               invoke self "allocateElement" returning aptr
               set address of anelement2 to aptr
           end-if
           perform until i > j
               compute realindex = (k - 1) * collectionstride + 1 +
                                   indexeddataoffset
               move instancedata(realindex:collectionstride) to
                    anelement2
               call "oopsresolve"
                          using by reference methodName
                                by reference methodImplementor
                                by value hashValue
                                by reference cacheblock
                        returning procPtr
               call procptr using bool
                                  anelement2
                                  anElement
               if istrue
                   compute i = k + 1
               else
                   compute j = k - 1
               end-if
               compute k = (i + j) / 2
           end-perform.
           add 1 to i
           subtract firstindex from i
           move i to anindex
           if collectionStride > 20
               invoke self "dealloc" using aptr
           end-if
           exit method.
       end method "indexForInserting".
      *>----------------------------------------------------------------
      *> PRIVATE. Describes the default order for instances of sorted
      *> collections. This method ensures that elements are stored in
      *> ascending order. Due to the fact that this collection has to
      *> work out what it is storing it would be more efficient if the
      *> user of SortedCollections implemented his own sort block. This
      *> would be particularly noticeable if the user was storing
      *> intrinsic types.
      *>----------------------------------------------------------------
       method-id. "defaultSortMethod".
       linkage section.
       01 anelement2.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 lnkbool                 pic x comp-5.
           88 lnkistrue           value 1.
           88 lnkisfalse          value 0.
       procedure division using anelement anelement2 returning lnkbool.
           if collectiontemplate equal to null
              invoke anElement as object "lessThanOrEqual"
                                       using anELement2
                                   returning lnkbool
           else
              invoke anElement as collectiontemplate
                   "lessThanOrEqualbyLengthValue"
                              using collectionstride anelement2
                       returning lnkbool
           end-if
           exit method.
       end method "defaultSortMethod".

      *>----------------------------------------------------------------
      *> Return OrderedCollection as the species of SortedCollections
      *>----------------------------------------------------------------
       method-id. "species".
       linkage section.
       01 lnkclass             usage object reference.

       procedure division returning lnkclass.
           set lnkClass to OrderedCollection
           exit method.
       end method "species".
      *>----------------------------------------------------------------
      *> Raise an exception as SortedCollections should not implement
      *> this method.
      *>----------------------------------------------------------------
       method-id. "insertAllAfter".
       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 anewcollection       usage object reference.
       01 acollection          usage object reference.

       procedure division using anindex acollection
                            returning anewcollection.
           invoke self "shouldNotImplement"
           exit method.
       end method "insertAllAfter".
      *>----------------------------------------------------------------
      *> Place anElement at every indexable  position of this
      *> SequencedCollection
      *>----------------------------------------------------------------
       method-id. "atAllPut".
       local-storage section.
       01 thissize             pic s9(9) comp-5.

       linkage section.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division  using anelement.
           invoke self "size" returning thissize
           invoke self "removefirst" using thissize
           perform thissize times
               invoke self "add" using anelement
           end-perform
           exit method.
       end method "atAllPut".
      *>----------------------------------------------------------------
      *> Replace all the occurrences of anElement between the indices
      *> fromIndex and toIndex with anElement2.
      *>----------------------------------------------------------------
       method-id. "replaceAll".
       local-storage section.
       01 i                    pic s9(9) comp-5.
       01 j                       pic s9(9) comp-5.

       linkage section.
       01 toindex              pic s9(9) comp-5.
       01 anelement2.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 fromindex            pic s9(9) comp-5.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anelement anelement2
                                          fromindex toindex.
           invoke self "firstIndexOf" using anelement
                                                 fromindex
                                                 toindex
                                       returning i
           perform until i = 0
               invoke self "removeAtIndex" using i
                                       returning anelement
               invoke self "add" using anelement2
               move i to j
               add 1 to j
               invoke self "firstIndexOf" using anelement
                                                     j
                                                     toindex
                                           returning i
           end-perform
           exit method.
       end method "replaceAll".
       end object.
       end class sortedcollection.
