      *>----------------------------------------------------------------
      *> A ValueSet stores non-ordered objects or intrinsics in a
      *> collection where duplicates are not allowed.
      *>
      *> Copyright (C) 1991,1996 Micro Focus Ltd.
      *>
      *>----------------------------------------------------------------
      $set localcount(200) linkcount(150) mfoo
       class-id. ValueSet data is protected
               inherits from collection with data.


        object section.
        class-control.
           collection is class "collectn"
           ValueSet is class "valueset"
           callback is class "callback".
       object-storage section.
      *>---------------------------------------------------------------
      *> PRIVATE. Method to create a ValueSet of any type.
      *>---------------------------------------------------------------
       method-id. "privatenew".
       local-storage section.
       01 newsize          pic s9(9) comp-5.

       linkage section.
       01 lnksize              pic s9(9) comp-5.
       01 lnkstride             pic s9(9) comp-5.
       01 acollection          usage object reference.
       01 lnktemplate          usage object reference.

       procedure division using lnksize lnktemplate lnkstride
                             returning acollection.
           compute newsize = lnksize * lnkstride
           compute newsize = newsize + (lnksize / 8) + 1
           invoke self "newIndexedInstance" using newsize
                                        returning acollection
           invoke acollection "initialize" using lnksize
                                                 lnktemplate
                                                 lnkstride
           exit method.
       end method "privatenew".

      *>---------------------------------------------------------------
      *> Return a new instance of ValueSet with a capability of holding
      *> lnkSize intrinsics of type defined by the class aTemplate.
      *>---------------------------------------------------------------
       method-id. "ofValues".
       local-storage section.
       01 astride              pic s9(9) comp-5.
       01 lssize               pic s9(9) comp-5.

       linkage section.
       01 lnktemplate          usage object reference.
       01 lnksize              pic s9(9) comp-5.
       01 acollection          usage object reference.

       procedure division using lnktemplate lnksize
                                            returning acollection.
           if lnksize < zero
               invoke self "invalidParameter" returning acollection
           else
               if lnksize = zero
                   move 1 to lssize
               else
                   move lnksize to lssize
               end-if
               invoke lnktemplate "sizeInBytes" returning astride
               invoke self "privateNew" using lssize
                                               lnktemplate
                                               astride
                                     returning acollection
           end-if
           exit method.
       end method "ofValues".
      *>----------------------------------------------------------------
      *> Return a new instance of ValueSet capable of holding lnkSize
      *> objects.
      *>----------------------------------------------------------------
       method-id. "ofReferences".
       local-storage section.
       01 astride              pic s9(9) comp-5.
       01 anObject             usage object reference.
       01 lssize               pic s9(9) comp-5.
       linkage section.
       01 lnksize              pic s9(9) comp-5.
       01 acollection          usage object reference.

       procedure division using lnksize returning acollection.
           if lnksize < zero
               invoke self "invalidParameter" returning acollection
           else
               if lnksize = zero
                   move 1 to lssize
               else
                   move lnksize to lssize
               end-if
               move objectReferenceSize to astride
               set anObject to null
               invoke self "privateNew" using lssize anObject astride
                                         returning acollection
           end-if
           exit method.
       end method "ofReferences".
       Object.

       object-storage section.

      *>>---------------------------------------------------------------
      *>> The total number of bytes required to store the empty flags
      *>>---------------------------------------------------------------

       01 emptybits pic x(4) comp-5 value 0.

      *>>---------------------------------------------------------------
      *>> The capacity (measured in objects being stored) that this
      *>> set is capable of holding.
      *>>---------------------------------------------------------------

       01 capacity  pic x(4) comp-5 value 0.
      *>---------------------------------------------------------------
      *> PRIVATE method used to initialize instance variables for new
      *> ValueSet's
      *>---------------------------------------------------------------
       method-id. "initialize".
       linkage section.
       01 lnkSize              pic s9(9) comp-5.
       01 lnkStride             pic s9(9) comp-5.
       01 lnkTemplate          usage object reference.

       procedure division using lnkSize lnkTemplate lnkStride.
           invoke super "initialize" using lnkSize lnkTemplate lnkStride
           compute emptybits = (lnkSize / 8) + 1
           move zero to numberelements
           move low-values to instancedata(indexeddataoffset + 1:
                                emptybits + storagerequirements)
           move storagerequirements to capacity
           divide collectionstride into capacity
           exit method.
       end method "initialize".
      *>----------------------------------------------------------------
      *> PRIVATE. Returns the element at the real index position
      *> anIndex. If anIndex is out of bounds an exception is raised.
      *>----------------------------------------------------------------
       method-id. "privateat".
       local-storage section.
       01 realindex            pic s9(9) comp-5.

       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 anelement2.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anindex returning anelement2.
           if anindex < 1 or anindex > capacity
               invoke self "errorInBounds" returning anelement2
               exit method
           else
               compute realindex = (anindex - 1) * collectionstride + 1
                                   + indexeddataoffset
               move instancedata(realindex:collectionstride) to
                   anelement2
               exit method
           end-if.
       end method "privateat".

      *>----------------------------------------------------------------
      *> PRIVATE. Returns true if anElement is at the real index
      *> position anIndex otherwise returns false.
      *>----------------------------------------------------------------
       method-id. "isat".
       local-storage section.
       01 aptr             pointer.

       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 lnkbool                 pic x comp-5.
           88 lnkistrue           value 1.
           88 lnkisfalse          value 0.
       01 anelement2.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anelement anindex
                          returning lnkbool.
           invoke self "isOccuppiedAt" using anindex
                                        returning lnkbool
           if lnkistrue
               invoke self "allocateElement" returning aptr
               set address of anelement2 to aptr

               invoke self "basicAt" using anindex returning anElement2
               if collectiontemplate equal to null
                   invoke anelement2 as object "equal" using anelement
                                  returning lnkbool
               else
                   if anelement2 = anelement
                       set lnkistrue to true
                   else
                       set lnkisfalse to true
                   end-if
               end-if
               invoke self "dealloc" using aptr
           end-if
           exit method.
       end method "isat".

      *>----------------------------------------------------------------
      *> Return the number of elements within this ValueSet.
      *>----------------------------------------------------------------
       method-id. "size".
       linkage section.
       01 acollectionsize      pic s9(9) comp-5.

       procedure division returning acollectionsize.
           move numberelements to acollectionsize
           exit method.
       end method "size".
      *>----------------------------------------------------------------
      *> Return true if this set contains anElement, otherwise return
      *> false.
      *>----------------------------------------------------------------
       method-id. "includes".
       local-storage section.
       01 i                    pic s9(9) comp-5.

       linkage section.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 lnkbool                 pic x comp-5.
           88 lnkistrue           value 1.
           88 lnkisfalse          value 0.

       procedure division using anelement returning lnkbool.
           invoke self "findElement" using anelement
                                      returning i
           invoke self "isOccuppiedAt" using i returning lnkbool
           exit method.
       end method "includes".
      *>----------------------------------------------------------------
      *> Return the number of elements anElement is equal to. The
      *> result of this is either one or zero because two equal elements
      *> cannot be stored in a set.
      *>----------------------------------------------------------------
       method-id. "occurrencesOf".
       local-storage section.
       01 bool                 pic x comp-5.
           88 istrue           value 1.
           88 isfalse          value 0.

       linkage section.
       01 anumber              pic s9(9) comp-5.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anelement returning anumber.
           invoke self "includes" using anelement
                                   returning bool
           if istrue
               move 1 to anumber
           else
               move zero to anumber
           end-if
           exit method.
       end method "occurrencesOf".
      *>----------------------------------------------------------------
      *> Return the number of elements this collection is capable of
      *> holding.
      *>----------------------------------------------------------------
       method-id. "capacity".
       linkage section.
       01 acollectionsize      pic s9(9) comp-5.

       procedure division returning acollectionsize.
           move capacity to acollectionsize
           exit method.
       end method "capacity".
      *>----------------------------------------------------------------
      *> Cause this ValueSet collection to become larger in order
      *> to store more members.
      *>----------------------------------------------------------------
       method-id. "grow".
       local-storage section.
       01 anewset              usage object reference.
       01 exceptionnumber      pic s9(9) comp-5.
       01 bool                 pic x comp-5.
          88 istrue           value 1.
          88 isfalse          value 0.
       01 oldsize              pic s9(9) comp-5.
       01 anobject         usage object reference.
       01 asize                pic s9(9) comp-5.
       01 newsize          pic s9(9) comp-5.

       procedure division.
           invoke self "basicCopy" returning anewset
           move capacity to oldsize
           add 5 to oldsize giving asize
           multiply 2 by asize
           move asize to storagerequirements capacity
           multiply collectionstride by storagerequirements
           compute emptybits = (asize / 8) + 1
           add emptybits to storagerequirements giving newsize
           invoke self "basicGrow" using newsize returning bool
           if isfalse
               invoke anewset "finalize" returning anobject
               move 32 to exceptionnumber
               invoke self "raiseException" using exceptionnumber
           else
               invoke self "replaceContentsWith" using anewset
               invoke anewset "finalize" returning anobject
           end-if
           exit method.
       end method "grow".
      *>----------------------------------------------------------------
      *> PRIVATE. Causes the whole contents of the receiver to be
      *> replaced with the contents of aCollection. The only use
      *> of this method is to perform rehashing after a grow.
      *>----------------------------------------------------------------
       method-id. "replaceContentsWith".
       local-storage section.
       01 amessageselector pic x(80).
       01 amessage2            usage object reference.

       linkage section.
       01 acollection          usage object reference.

       procedure division using acollection.
           move low-values to instancedata(indexeddataoffset + 1:
                                      storagerequirements + emptybits)
           move zero to numberelements
           move "add" to amessageselector
           invoke callback "new" using self amessageselector
                                    returning amessage2
           invoke acollection "doWithOutCopy" using amessage2
           invoke amessage2 "finalize" returning amessage2
           exit method.
       end method "replaceContentsWith".
      *>----------------------------------------------------------------
      *> Return a new set containing the members of this set
      *> concatenated with the members of aCollection.
      *>----------------------------------------------------------------
       method-id. "concat".
       local-storage section.
       01 bool                 pic x comp-5.
           88 istrue           value 1.
           88 isfalse          value 0.

       linkage section.
       01 anewcollection       usage object reference.
       01 acollection          usage object reference.

       procedure division using acollection
                            returning anewcollection.
           invoke self "isCompatible" using acollection
                                       returning bool
           if istrue
               invoke self "basicCopy" returning anewcollection
               invoke anewcollection "addAll" using acollection
           else
               invoke self "compatibilityError"
                       returning aNewCollection
           end-if
           exit method.
       end method "concat".
      *>----------------------------------------------------------------
      *> Iterate over the members of the set sending aMessage for each
      *> element, passing that element as a parameter.
      *> aMessage should be a CallBack pointing to a method which
      *> expects an element as a parameter.
      *> For more information see the topic Collection Iterator Methods
      *> in the on-line documentation.
      *>----------------------------------------------------------------
       method-id. "doWithOutCopy".
       local-storage section.
       01 ooreturnptr          usage pointer.
       01 bool                 pic x comp-5.
          88 istrue           value 1.
          88 isfalse          value 0.
       01 j                    pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 thissize             pic s9(9) comp-5.

       linkage section.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 amessage             usage object reference.
       procedure division using amessage.
           invoke self "size" returning thissize
           set oonoquit to true

           invoke self "allocateElement" returning ooreturnptr
           move zero to i
           set address of anelement to ooreturnPtr
           perform varying j from 1 by 1 until i = thissize or ooquit
               invoke self "isEmptyAt" using j returning bool
               if isfalse
                   invoke self "basicAt" using j returning anElement
                   invoke amessage "invoke" using anElement
                   add 1 to i
               end-if
           end-perform
           set oonoquit to true
           invoke self "dealloc" using ooreturnptr
           exit method.
       end method "doWithOutCopy".
      *>----------------------------------------------------------------
      *> Send aMessage for each element of this value set in turn,
      *> passing that element as a parameter.
      *> aMessage should be a CallBack pointing to a method which
      *> expects an element as a parameter.
      *> For more information see the topic Collection Iterator Methods
      *> in the on-line documentation.
      *>----------------------------------------------------------------
       method-id. "do".
       local-storage section.
       01 ooreturnptr          usage pointer.
       01 bool                 pic x comp-5.
          88 istrue           value 1.
          88 isfalse          value 0.
       01 j                    pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 thissize             pic s9(9) comp-5.
       01 aCopy                usage object reference.

       linkage section.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.
       01 amessage             usage object reference.
       procedure division using amessage.
           invoke self "size" returning thissize
           set oonoquit to true

           invoke self "allocateElement" returning ooreturnptr
           move zero to i
           set address of anelement to ooreturnPtr
           invoke self "copy" returning aCopy
           perform varying j from 1 by 1 until i = thissize or ooquit
               invoke aCopy "isEmptyAt" using j returning bool
               if isfalse
                   invoke aCopy "basicAt" using j returning anElement
                   invoke amessage "invoke" using anElement
                   add 1 to i
               end-if
           end-perform
           invoke aCopy "finalize" returning aCopy
           set oonoquit to true
           invoke self "dealloc" using ooreturnptr
           exit method.
       end method "do".
      *>----------------------------------------------------------------
      *> Behaves exactly as the method "do" as ValueSet does not store
      *> elements in any predefined order.
      *>----------------------------------------------------------------
       method-id. "reverseDo".
       linkage section.
       01 amessage             usage object reference.

       procedure division using amessage.
           invoke self "do" using amessage
           exit method.
       end method "reverseDo".
      *>---------------------------------------------------------------
      *> PRIVATE. Return address of element at position anIndex
      *>---------------------------------------------------------------
       method-id. "basicAt".
       local-storage section.
       01 realindex           pic s9(9) comp-5.
       01 anAddress           usage pointer.
       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 anElement.
           03 aByte pic x occurs 1 to maxNumberBytes depending on
                               collectionStride.
       01 anElement2.
           03 aByte pic x occurs 1 to maxNumberBytes depending on
                               collectionStride.

       procedure division using anindex returning anElement.
           subtract 1 from anIndex giving realIndex
           multiply collectionStride by realIndex
           add indexedDataOffset to realIndex

           set anaddress to address of instancedata
           set anaddress up by realindex
           set address of anElement2 to anAddress

           move anElement2 to anElement
           exit method.
       end method "basicAt".
      *>---------------------------------------------------------------
      *> PRIVATE. Returns the index of the anElement or an index for
      *> where anElement can be stored if it doesn't exist
      *>---------------------------------------------------------------
       method-id. "findElement".
       local-storage section.
       01 temp                 pic s9(9) comp-5.
       01 bool                 pic x comp-5.
          88 istrue           value 1.
          88 isfalse          value 0.
       01 asize                pic s9(9) comp-5.
       01 hashvalue            pic s9(9) comp-5.

       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anelement returning anindex.
           move capacity to asize
           if collectiontemplate equal to null
               invoke anelement as object
                   "hash" returning hashvalue
           else
               invoke anelement as collectiontemplate
                               "hash" returning hashvalue
           end-if
           move hashvalue to anindex
           set isfalse to true
           perform with test after until istrue
               divide anindex by asize giving temp
                                      remainder anindex
               add 1 to anindex
               invoke self "isEmptyAt" using anindex returning bool
               if isfalse
                   invoke self "isAt" using anelement anindex
                                   returning bool
               end-if
           end-perform
           exit method.
       end method "findElement".

      *>---------------------------------------------------------------
      *> Add anElement to this ValueSet.
      *>---------------------------------------------------------------
       method-id. "add".
       local-storage section.
       01 realindex            pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 bool                 pic x comp-5.
          88 istrue            value 1.
          88 isfalse           value 0.
       01 growthLimit         pic x(4) comp-5.

       linkage section.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anelement.
           multiply capacity by 7 giving growthLimit
           divide 10 into growthLimit
           if growthLimit <= numberelements
               invoke self "grow"
           end-if
           invoke self "findELement" using anelement returning i
           invoke self "isEmptyAt" using i returning bool
           if istrue
               compute realindex = (i - 1) * collectionstride + 1 +
                                   indexeddataoffset
               move anelement to
                   instancedata(realindex:collectionstride)
               invoke self "setOccuppied" using i
               add 1 to numberelements
           end-if
           exit method.
       end method "add".
      *>---------------------------------------------------------------
      *> Remove anElement from this ValueSet's members. If anElement
      *> does not exist then raise an exception. Return the element
      *> that has been removed.
      *>---------------------------------------------------------------
       method-id. "remove".
       local-storage section.
       01 realindex            pic s9(9) comp-5.
       01 exceptionnumber  pic s9(9) comp-5.
       01 bool                 pic x comp-5.
          88 istrue           value 1.
          88 isfalse          value 0.
       01 i                    pic s9(9) comp-5.

       linkage section.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anelement.
           invoke self "findElement" using anelement returning i
           invoke self "isOccuppiedAt" using i returning bool
           if isfalse
               move 12 to exceptionnumber
               invoke self "raiseException" using exceptionnumber
           else
               compute realindex = (i - 1) * collectionstride + 1 +
                                   indexeddataoffset
               move low-values to
                      instancedata(realindex:collectionstride)
               subtract 1 from numberelements
               invoke self "setEmpty" using i
               invoke self "fixCollisionsFrom" using i
           end-if
           exit method.
       end method "remove".
      *>---------------------------------------------------------------
      *> PRIVATE. Fixes all the collisions after an element has been
      *> removed.
      *>---------------------------------------------------------------
       method-id. "fixCollisionsFrom".
       local-storage section.
       01 temp                 pic s9(9) comp-5.
       01 realindex            pic s9(9) comp-5.
       01 bool                 pic x comp-5.
          88 istrue           value 1.
          88 isfalse          value 0.
       01 aptr                 pointer.
       01 j                    pic s9(9) comp-5.
       01 i                    pic s9(9) comp-5.
       01 asize                pic s9(9) comp-5.

       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 anelement.
           03 abyte            pic x  occurs 0 to maxnumberbytes
                                     depending on collectionstride.

       procedure division using anindex.
           move capacity to asize
           divide anindex by asize giving temp remainder i
           add 1 to i
           invoke self "isEmptyat" using i returning bool
           invoke self "allocateElement" returning aptr
           set address of anelement to aptr
           perform until istrue
               invoke self "basicat" using i returning anElement
               invoke self "findElement" using anelement returning j
               if j not = i
                   compute realindex = (j - 1) * collectionstride + 1 +
                                       indexeddataoffset
                   move anelement to
                           instancedata (realindex:collectionstride)
                   invoke self "setoccuppied" using j
                   compute realindex = (i - 1) * collectionstride + 1 +
                                       indexeddataoffset
                   move low-values to
                       instancedata(realindex:collectionstride)
                   invoke self "setempty" using i
               end-if
               divide i by asize giving temp remainder i
               add 1 to i
               invoke self "isEmptyat" using i returning bool
           end-perform
           invoke self "dealloc" using aptr
           exit method.
       end method "fixCollisionsFrom".

      *>----------------------------------------------------------------
      *> Return a new ValueSet which containing those elements which
      *> are in this ValueSet but not in aCollection.
      *>----------------------------------------------------------------
       method-id. "minus".
       local-storage section.
       01 amessageselector     pic x(80).
       01 amessage2            usage object reference.

       linkage section.
       01 anewcollection       usage object reference.
       01 acollection          usage object reference.

       procedure division using acollection
                           returning anewcollection.
           move "includes" to amessageselector
           invoke callback "new" using acollection amessageselector
                                     returning amessage2
           invoke self "reject" using amessage2
                                   returning anewcollection
           exit method.
       end method "minus".
      *>---------------------------------------------------------------
      *> PRIVATE. Mark the position anIndex as Occuppied
      *>---------------------------------------------------------------
       method-id. "setOccuppied".
       local-storage section.
       01 i                    pic s9(9) comp-5.
       01 abit             pic x comp-5.
       01 anoffset         pic s9(9) comp-5.
       01 amask            pic x comp-x.

       linkage section.
       01 anindex              pic s9(9) comp-5.

       procedure division using anindex.
           divide anindex by 8 giving anoffset remainder abit
           add 1 to anoffset
           move 1 to amask
           perform abit times
               multiply 2 by amask
           end-perform
           move 1 to i
           call "CBL_OR" using amask
                    instancedata(anoffset + indexeddataoffset +
                                   storagerequirements:i)
                    by value 1
           exit method.
       end method "setOccuppied".

      *>---------------------------------------------------------------
      *> PRIVATE. Mark the position anIndex as empty
      *>---------------------------------------------------------------
       method-id. "setEmpty".
       working-storage section.
       01 afullmask        pic x comp-x value h"FF".

       local-storage section.
       01 i                pic s9(9) comp-5.
       01 abit             pic x comp-5.
       01 anoffset         pic s9(9) comp-5.
       01 amask            pic x comp-x.

       linkage section.
       01 anindex          pic s9(9) comp-5.

       procedure division using anindex.
           divide anindex by 8 giving anoffset remainder abit
           add 1 to anoffset
           move 1 to amask
           perform abit times
               multiply 2 by amask
           end-perform
           call "CBL_XOR" using afullmask amask by value 1
           move 1 to i
           call "CBL_AND" using amask
                    instancedata(anoffset + indexeddataoffset +
                                   storagerequirements:i)
                             by value 1
           exit method.
       end method "setEmpty".

      *>---------------------------------------------------------------
      *> PRIVATE. If the location indicated by anIndex is empty then
      *> return true otherwise return false.
      *>---------------------------------------------------------------
       method-id. "isEmptyAt".
       local-storage section.
       01 anemptybyte      pic x comp-x.
       01 anebyte redefines anemptybyte pic x.
       01 i                pic s9(9) comp-5.
       01 amask            pic x comp-x.
       01 abit             pic x comp-5.
       01 anoffset         pic s9(9) comp-5.

       linkage section.
       01 anindex              pic s9(9) comp-5.
       01 lnkbool                 pic x comp-5.
           88 lnkistrue           value 1.
           88 lnkisfalse          value 0.

       procedure division using anindex returning lnkbool.
           divide anindex by 8 giving anoffset remainder abit
           add 1 to anoffset
           move 1 to amask
           perform abit times
               multiply 2 by amask
           end-perform
           move 1 to i
           move instancedata(anoffset + indexeddataoffset +
                          storagerequirements:i) to anebyte
           call "CBL_AND" using amask anemptybyte by value 1
           if anemptybyte > 0
               set lnkisfalse to true
           else
               set lnkistrue to true
           end-if
           exit method.
       end method "isEmptyAt".

      *>---------------------------------------------------------------
      *> PRIVATE. If the location indicated by anIndex is occuppied then
      *> return true otherwise return false.
      *>---------------------------------------------------------------
       method-id. "isOccuppiedAt".
       local-storage section.
       01 anemptybyte      pic x comp-x.
       01 anebyte          redefines anemptybyte pic x.
       01 i                pic s9(9) comp-5.
       01 amask            pic x comp-x.
       01 abit             pic x comp-5.
       01 anoffset         pic s9(9) comp-5.

       linkage section.
       01 anindex           pic s9(9) comp-5.
       01 lnkbool           pic x comp-5.
           88 lnkistrue     value 1.
           88 lnkisfalse    value 0.

       procedure division using anindex returning lnkbool.
           divide anindex by 8 giving anoffset remainder abit
           add 1 to anoffset
           move 1 to amask
           perform abit times
               multiply 2 by amask
           end-perform
           move 1 to i
           move instancedata(anoffset + indexeddataoffset +
                                   storagerequirements:i) to anebyte
           call "CBL_AND" using amask anemptybyte by value 1
           if anemptybyte > 0
               set lnkistrue to true
           else
               set lnkisfalse to true
           end-if
           exit method.
       end method "isOccuppiedAt".

      *>----------------------------------------------------------------
      *> PRIVATE. Called when object becomes a component
      *>----------------------------------------------------------------
       method-id. "becomeComponent".
       local-storage section.
       01 lsDataPointer       pointer.
       01 lsIndexedDataSize   pic s9(9) comp-5.
       01 lsDynamicDataOffset pic s9(9) comp-5.
       01 lsInstanceDataSize  pic s9(9) comp-5.
       01 lsAddOffset         pic s9(9) comp-5.
       01 lsDestOffset        pic s9(9) comp-5.
       01 lsSrcOffset         pic s9(9) comp-5.
       01 i                   pic s9(9) comp-5.
        linkage section.
        01 lnkId                        pic x.
       procedure division.
           call "oopsgetaddress" using self
                                       lsDataPointer
                                       lsInstanceDataSize
           call "oopsindexeddataoffset" using selfClass
                              returning lsDynamicDataOffset

           compute lsAddOffset = lsInstanceDataSize -
                                 lsDynamicDataOffset -
                                 storageRequirements - emptyBits
           set address of lnkID to lsDataPointer
           compute lsDestOffset = indexedDataOffset + lsAddOffset 
           add emptyBits to storageRequirements giving i 
           perform varying i from i by -1 until i = 0
               move lnkID(indexedDataOffset + i: 1) to
                  lnkID(lsDestOffset + i : 1)
           end-perform
           add lsAddOffset to indexedDataOffset
           invoke self "basicbecomeComponent"
           exit method.
       end method "becomeComponent".

      *>----------------------------------------------------------------
      *> PRIVATE. Returns the storage requirements for this collection.
      *>----------------------------------------------------------------
       method-id. "getStorageRequirements".
       linkage section.
       01 lnkRequirements  pic x(4) comp-5.
       procedure division returning lnkRequirements.
           move storageRequirements to lnkRequirements
           add emptybits to lnkRequirements
           exit method.
       end method "getStorageRequirements".

       end object.
       end class ValueSet.
