/************************************************************************
 *
 *	Copyright (C) 1984-1998 Micro Focus Ltd.
 *
 *	All  rights reserved.  No part of this program or  publication
 *	may  be  reproduced,  transmitted,  transcribed,  stored  in a
 *	retrieval system,  or translated into any language or computer
 *	language, in any form or by any means, electronic, mechanical,
 *	magnetic,  optical, chemical, manual or otherwise, without the
 *	prior written permission of:
 *
 *	Micro Focus Ltd,
 *	26 West Street, Newbury, Berkshire, United Kingdom.             
 *	(0635) 32646
 *
 *	Phase 9a Base Run Time System
 *
 *	@(#)gpgetput.h	1.9
 *
 *	Last Modified on 98/06/04 at 21:30:59
 *
 ************************************************************************/

/*
 *    the data pickers and unpickers.
 *	the structures are very machine dependent.
 *	the get macros must be of type unsigned, and prevent extension.
 *	the globals used in macros are in runmch.h
 *    the macros could be better tuned to your target machine...
 *	these represent a compromise for several machines and compilers.
 *    the define telling the cpu is in 'control.h', which must have already
 *	been included, or the compile line must define the right one.
 */

#include	"mf_types.h"

/* @(#)ugetput.sed	1.1	*/

#define	get3ws	mFt_get3ws
#define	get4ws	mFt_get4ws
#define	put3ws	mFt_put3ws
#define	put4ws	mFt_put4ws
#define	execerr	mFt_execerr
#define	ALIGN1
#define	ALIGN8
#define RS6000
#define COBOLORDER
#define INLINE_4WS

/* Below are the structures that define how the COBOL order bytes are mapped
   onto machine words for the various supported byte orderings.
*/

#ifdef COBOLORDER
struct w2b	{ mf_uns8	mFt_hibyte,	mFt_lobyte;	};
struct l4b	{ mf_uns8	hwhb,	hwlb,	lwhb,	lwlb;	};
#else
#  ifdef SWAPORDER
struct w2b	{ mf_uns8	mFt_lobyte,	mFt_hibyte;	};
struct l4b	{ mf_uns8	lwlb,	lwhb,	hwlb,	hwhb;	};
#  else
#    ifdef ODDORDER
struct w2b	{ mf_uns8	mFt_hibyte,	mFt_lobyte;	};
struct l4b	{ mf_uns8	hwlb,	hwhb,	lwlb,	lwhb;	};
#    else
	Compilation error - Byte ordering unknown.
#    endif
#  endif
#endif

/* These unions allow us to manipulate using the above structures byte-by-byte
   and pull out the result as a machine-order word.
*/
extern union word2byte
{	mf_uns16	w;
	struct w2b	b;
} zu;

union long4byte
{	mf_uns32	l;
	struct l4b	b;
};

extern mf_uns8	 	*mFt_zp;	/* g.p. char pointer  */
extern mf_uns8	 	zc;		/* g.p. char */
extern union long4byte	zl;

/******************************************************************************/
/********* Below are the macros that may manipulate the unions above **********/
/******************************************************************************/

/****************************** get/put1 macros *******************************/
/*** These are always the same no matter what the chips characteristics are ***/
/******************************************************************************/

		/* COMP/COMP-X Macros	*/
#define	get1ws(x)	(*(mf_uns8 *)(x))
#define	put1ws(a,v)	(*(mf_uns8 *)(a) = (mf_uns8)(v))

		/* COMP-5 Macros	*/
#define	get1ws5(x)	(*(mf_uns8 *)(x))
#define	put1ws5(a,v)	(*(mf_uns8 *)(a) = (mf_uns8)(v))

		/* INT-code Macro	*/
#define get1ic()	(*(mf_uns8 *)mFt_pc++)


/******************************************************************************/
/*** If COBOLORDER, then comp-5 uses same function/macro as Cobol order     ***/
/******************************************************************************/
#ifdef COBOLORDER
#define	get2ws5(x)		get2ws(x)
#define	mFt_get3ws5(x)		mFt_get3ws(x)
#define	mFt_get4ws5(x)		mFt_get4ws(x)
#define	put2ws5(a, v)		put2ws(a, v)
#define	mFt_put3ws5(a, v)	mFt_put3ws(a, v)
#define	mFt_put4ws5(a, v)	mFt_put4ws(a, v)
#endif

#if defined(ALPHA_NT) && !defined(ALIGN1) && !defined(COBOLORDER)
#define get2ws  mFt_get2ws
#define put2ws  mFt_put2ws
#endif

/***************************** Multi-byte macros ******************************/

#ifdef ALIGN1
	/* For misalignment-tolerant chips, we can just use pointer
	   dereferencing for machine order items.  */
#define	mFt_getptr(x)		((char *)(*(void **)x))
#define	mFt_putptr(a, v)	((char *)(*(void **)(a) = (v)))

#  ifdef COBOLORDER
	/* Special case using simple pointer dereferencing for those chips
	   which are both COBOL byte order and tolerant of misalignment.  */

			/* COMP/COMP-X Macros	*/
#define	get2ws(x)		(*(mf_uns16 *)(x))
#define	mFt_get3ws(x)		(mf_uns32)((get2ws(x) << 8) | get1ws((x) + 2))
#define	mFt_get4ws(x)		(*(mf_uns32 *)(x))

#define	put2ws(a, v)		(*(mf_uns16 *)(a) = (mf_uns16)(v))
#define mFt_put3ws(a, v)	(zl.l = (mf_uns32)(v),			\
				*((mf_uns8 *)(a)) = zl.b.hwlb,		\
				*((mf_uns8 *)(a)+1) = zl.b.lwhb,	\
				*((mf_uns8 *)(a)+2) = zl.b.lwlb)
#define	mFt_put4ws(a, v)	(*(mf_uns32 *)(a) = (mf_uns32)(v))

			/* INT-code Macros	*/
#define	get2ic()	(mFt_pc += 2, *(mf_uns16 *)(mFt_pc - 2))

#  else
			/* COMP-5/POINTER Macros */
#define	get2ws5(x)		(*(mf_uns16 *)(x))
#define	mFt_get4ws5(x)		(*(mf_uns32 *)(x))

#define	put2ws5(a, v)		(*(mf_uns16 *)(a) = (mf_uns16)(v))
#define	mFt_put4ws5(a, v)	(*(mf_uns32 *)(a) = (mf_uns32)(v))

#  endif
#endif


/* Default definitions used if no other definition provided.
 * If you have a get??? macro, then there is a put??? macro */


#ifndef get2ic
#define get2ic()	 (zu.b.mFt_hibyte = *mFt_pc++,			\
			  zu.b.mFt_lobyte = *mFt_pc++,			\
			  zu.w)
#endif

#ifndef	get2ws
#define get2ws(a) ((mf_uns16)((((mf_uns16)(*((mf_uns8 *)(a)))) << (mf_uns16)8) | *((mf_uns8 *)(a)+1)))
#endif

#ifndef	put2ws
#define put2ws(a, v)	(zu.w = (unsigned)(v),				\
			 *((mf_uns8 *)(a)) = zu.b.mFt_hibyte,		\
			 *((mf_uns8 *)(a) + 1) = zu.b.mFt_lobyte)
#endif


#ifndef get2ws5
#define get2ws5(a)	(zu.b.mFt_lobyte = *((mf_uns8 *)(a)),		\
			 zu.b.mFt_hibyte = *((mf_uns8 *)(a) + 1),	\
			 zu.w)

#define put2ws5(a, v)	(zu.w = (unsigned)(v),				\
			 *((mf_uns8 *)(a)) = zu.b.mFt_lobyte,		\
			 *((mf_uns8 *)(a) + 1) = zu.b.mFt_hibyte)
#endif /* get2ws5 */


#ifndef mFt_get4ws
# ifdef INLINE_4WS
#define mFt_get4ws(a)	(zl.b.hwhb = *((mf_uns8 *)(a)),			\
			 zl.b.hwlb = *((mf_uns8 *)(a) + 1),		\
			 zl.b.lwhb = *((mf_uns8 *)(a) + 2),		\
			 zl.b.lwlb = *((mf_uns8 *)(a) + 3),		\
			 zl.l)

#define mFt_put4ws(a, v)	(zl.l = (mf_uns32)(v),			\
				 *((mf_uns8*)(a))  = zl.b.hwhb,		\
				 *((mf_uns8*)(a) + 1) = zl.b.hwlb,	\
				 *((mf_uns8*)(a) + 2) = zl.b.lwhb,	\
				 *((mf_uns8*)(a) + 3) = zl.b.lwlb)
#  else
mf_uns32 mFt_get4ws();
void mFt_put4ws();
#  endif	/* INLINE_4WS	*/
#endif		/* mFt_get4ws	*/


#ifndef mFt_get4ws5
# ifdef INLINE_4WS
#define mFt_get4ws5(a)		(memcpy((char *)&zl.l, (mf_uns8 *)(a), 4), zl.l)
#define mFt_put4ws5(a, v)	(zl.l = (mf_uns32)(v),			\
				 memcpy((char *)(a), (char *)&zl.l, 4))
#  else
mf_uns32 mFt_get4ws5();
void mFt_put4ws5();
#  endif	/* INLINE_4WS	*/
#endif		/* mFt_get4ws5	*/


#ifndef mFt_getptr
#  ifdef INLINE_4WS
#define mFt_getptr(a)		(memcpy(&mFt_zp, (char *)(a), sizeof(mFt_zp)), \
				 (char *)mFt_zp)

#define mFt_putptr(a,v)  	(mFt_zp = (mf_uns8 *)(v), 		\
				 memcpy((char *)(a), &mFt_zp, sizeof(a)))
#  else
char *mFt_getptr();
void mFt_putptr();
#  endif	/* INLINE_4WS	*/
#endif		/* mFt_getptr	*/


#ifndef mFt_get3ws
mf_uns32 mFt_get3ws();
void mFt_put3ws();
#endif


#ifndef mFt_get3ws5
mf_uns32 mFt_get3ws5();
void mFt_put3ws5();
#endif

/*** end of source module gpgetput.h ***/
